<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Speechkit
 * @subpackage Speechkit/admin
 * @author     Kostas Vaggelakos <kostas@speechkit.io>
 * @link       https://speechkit.io
 * @since      1.0.0
 */
class Speechkit_Admin
{
    /**
     * The ID of this plugin.
     *
     * @since  1.0.0
     * @access private
     * @var    string  $plugin_name The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since  1.0.0
     * @access private
     * @var    string  $version The current version of this plugin.
     */
    private $version;

    /**
     * The SpeechKit Dashboard URL.
     *
     * Override using SPEECHKIT_DASHBOARD_URL in wp-config.php.
     *
     * @since  2.9.1
     * @access private
     * @var    string  $dashboard_url SpeechKit Dashboard URL
     */
    private $dashboard_url = 'https://my.speechkit.io';

    /**
     * Autoregenerate audio when Posts are updated?
     *
     * Override using SPEECHKIT_AUTOREGENERATE in wp-config.php.
     *
     * @since 2.14.0
     * @var   boolean $autoregenerate Autoregenerate audio
     */
    public $autoregenerate = true;

    /**
     * State constants.
     *
     * The Podcast state, returned to us from the API. Possible values are:
     *
     * - unprocessed: New, hasn't been processed yet (in the queue)
     * - processing:  Currently being processed
     * - processed:   Fully processed and ready to listen
     * - error:       Failed, an Error occurred in the API while processing
     * - skipped:     Marked as skipped from source, this podcast won't be processed
     * - handled:     An internal API state, not relevant to WordPress
     *
     * Deprecated states:
     *
     * - scheduled: On hold, a state change is required to continue processing
     */
    const STATE_UNPROCESSED = 'unprocessed';
    const STATE_PROCESSING  = 'processing';
    const STATE_PROCESSED   = 'processed';
    const STATE_ERROR       = 'error';
    const STATE_SKIPPED     = 'skipped';
    const STATE_HANDLED     = 'handled';
    const STATE_SCHEDULED   = 'scheduled'; // deprecated in v2.9.5

    /**
     * Status constants
     */
    const STATUS_NEW              = 'New';              // Presented as "Processing"
    const STATUS_UPDATED          = 'Updated';          // Presented as "Processing"
    const STATUS_PROCESSING       = 'Processing';
    const STATUS_PROCESSED        = 'Processed';
    const STATUS_ERROR            = 'Error';
    const STATUS_FAILED           = 'Failed';
    const STATUS_PAYMENT_REQUIRED = 'Payment Required';
    const STATUS_SCHEDULED        = 'Scheduled';        // deprecated in v2.9.5

    /**
     * Number of retries.
     */
    const RETRIES = 15;

    /**
     * Initialize the class and set its properties.
     *
     * @since 1.0.0
     *
     * @param string $plugin_name The name of this plugin.
     * @param string $version     The version of this plugin.
     *
     * @return void
     */
    public function __construct($plugin_name, $version)
    {
        $speechkit_id = $this->get_setting('speechkit_id');

        $this->plugin_name = $plugin_name;

        // Using old version as a default value. All 2.5+ version will have correct
        // value present in DB, so we need to show older version just for those
        // old installs.
        $this->version = $version;

        // The Dashboard URL can be overridden in wp-config.php
        if (defined('SPEECHKIT_DASHBOARD_URL') && filter_var(SPEECHKIT_DASHBOARD_URL, FILTER_SANITIZE_URL)) {
            $this->dashboard_url = SPEECHKIT_DASHBOARD_URL;
        }

        // Autoregenerate can be overridden in wp-config.php
        if (defined('SPEECHKIT_AUTOREGENERATE')) {
            $this->autoregenerate = SPEECHKIT_AUTOREGENERATE;
        }

        $this->api = new SpeechkitAPI(
            $this->version,
            $speechkit_id
        );
    }

    /**
     * Get the valid post statuses for our WP_Query params.
     * - We will only send a post to the SpeechKit API if it has one of these statuses.
     *
     * @link https://wordpress.org/support/article/post-status/
     *
     * @static
     * @return array Array of strings
     **/
    public static function valid_post_statuses()
    {
        return array(
            'future',
            'publish',
            // 'pending', // todo we may want to process post status == 'pending', but remember it would use credits
            'private',
        );
    }

    /**
     * Get the post types which SpeechKit supports.
     *
     * Specifically, any post type which does not have 'custom-fields' in the
     * 'supports' array will not work with SpeechKit.
     *
     * @static
     * @return array Array of WP_Post_Types
     **/
    public static function get_supported_post_types()
    {
        $supported_post_types = array();

        $all_post_types = get_post_types(array(), 'objects');

        foreach ($all_post_types as $name => $post_type) {
            if (post_type_supports($name, 'custom-fields')) {
                $supported_post_types[$name] = $post_type;
            }
        }

        return $supported_post_types;
    }

    /**
     * Get the post types which are allowed to be selected.
     * We forbid some post types to be selected, because they
     * are unsuitable for audio generation.
     *
     * @return array Array of WP_Post_Type
     **/
    public static function get_allowed_post_types()
    {
        $forbidden = array(
            'attachment',
            'custom_css',
            'customize_changeset',
            'nav_menu_item',
            'oembed_cache',
            'revision',
            'user_request',
            'wp_block',
            'wp_template',
        );

        // Post types without Custom Fields are not supported
        $supported_post_types = self::get_supported_post_types();

        // Copy array values to keys, for simple diff comparison below
        $forbidden = array_combine($forbidden, $forbidden);

        return array_diff_key($supported_post_types, $forbidden);
    }

    /**
     * Get the post types selected to work with SpeechKit.
     *
     * @static
     * @return array Array of WP_Post_Types
     **/
    public static function selected_post_types()
    {
        $selected_post_types = Speechkit_Admin::get_setting('speechkit_select_post_types');

        if (false === is_array($selected_post_types)) {
            $selected_post_types = array();
        }

        // Post types without Custom Fields are not supported
        $supported_post_types = self::get_supported_post_types();

        // Copy array values to keys, for simple diff comparison below
        $selected_post_types = array_combine($selected_post_types, $selected_post_types);

        return array_intersect_key($supported_post_types, $selected_post_types);
    }

    /**
     * Get the names of the post types selected to work with SpeechKit.
     *
     * @return array Array of WP_Post_Type name properties
     **/
    public static function selected_post_type_names()
    {
        return array_keys(self::selected_post_types());
    }

    /**
     * Add items to the WordPress admin menu
     *
     * @return void
     */
    public function add_menu()
    {
        // Settings > SpeechKit
        add_options_page(__('SpeechKit', 'speechkit'), __('SpeechKit', 'speechkit'), 'manage_options', 'speechkit', array($this, 'create_admin_interface'));
    }

    /**
     * Show the classic Meta Box when the classic editor is active.
     *
     * @return void
     */
    public function add_meta_box()
    {
        $post_types = self::selected_post_type_names();
        if (!empty($post_types)) {
            add_meta_box(
                'speechkit',
                __('SpeechKit', 'speechkit'),
                array($this, 'create_admin_meta_box'),
                $post_types,
                'side',
                'default',
                array('__back_compat_meta_box' => true)
            );
        }
    }

    /**
     * Register the stylesheets for the admin area.
     *
     * @since  1.0.0
     *
     * @param string $hook Page hook
     *
     * @return void
     */
    public function enqueue_styles($hook)
    {
        // Always add the metabox css, because we don't know where it's added
        wp_enqueue_style($this->plugin_name . '-metabox', plugin_dir_url(__FILE__) . 'css/speechkit-admin-metabox.css', array(), $this->version, 'all');
    }

    public function setup_cron_schedules($schedules)
    {
        if (!isset($schedules['1min'])) {
            $schedules['1min'] = array(
                'interval' => 1 * 60,
                'display'  => __('Once every minute', 'speechkit'),
            );
        }
        return $schedules;
    }

    public function add_settings_link_to_plugin_page($links)
    {
        $links[] = '<a href="' .
            esc_url(admin_url('options-general.php?page=speechkit')) .
            '">' . __('Settings', 'speechkit') . '</a>';
        return $links;
    }

    /**
     * Register the JavaScript for the admin area.
     *
     * @since  1.0.0
     *
     * @param string $hook Page hook
     *
     * @return void
     */
    public function enqueue_scripts($hook)
    {
        // Enqueue our scripts on the post/page editing screens
        // (this includes Custom Post Types too)
        $screens = array(
            'post.php',
            'post-new.php',
            'page.php',
            'page-new.php',
        );

        if (in_array($hook, $screens)) {

            // Register the script
            wp_register_script('speechkit-metabox', plugin_dir_url(__FILE__) . 'js/classic-editor/speechkit-admin-metabox.js', array('jquery', 'underscore'), $this->version, true);

            // Localize the script with new data
            $translations = array(
                'error'       => __("Something went wrong. This is probably SpeechKit's fault. Please try again later", 'speechkit'),
                'enabled'     => __('Hide Audio Player', 'speechkit'),
                'disabled'    => __('Show Audio Player', 'speechkit'),
                'proccessing' => __('Proccesing Audio', 'speechkit'),
                'tpl'         => array(
                    'New'              => $this->tpl_processing(self::STATUS_NEW),
                    'Updated'          => $this->tpl_processing(self::STATUS_UPDATED),
                    'Processing'       => $this->tpl_processing(),
                    'Processed'        => $this->tpl_processed(),
                    'Error'            => $this->tpl_error(),
                    'Payment Required' => $this->tpl_payment_required(),
                    'Scheduled'        => $this->tpl_processed(self::STATUS_PROCESSED), // deprecated in v2.9.5
                ),
                'gutenberg' => $this->is_gutenberg_page(),
                'selectedCategories' => $this->get_setting('speechkit_selected_categories'),
            );

            wp_localize_script('speechkit-metabox', 'speechkitData', $translations);

            // Enqueued script with localized data.
            wp_enqueue_script('speechkit-metabox');
        }

        if ($hook === 'settings_page_speechkit') {
            // Register the script
            wp_register_script('speechkit-settings', plugin_dir_url(__FILE__) . 'js/classic-editor/speechkit-settings.js', array('jquery', 'underscore'), $this->version, true);

            // Enqueued script
            wp_enqueue_script('speechkit-settings');
        }
    }

    /**
     * Check to see if the Gutenberg Editor is being used.
     *
     * @link https://wordpress.stackexchange.com/a/324866
     */
    public function is_gutenberg_page()
    {
        if (function_exists('is_gutenberg_page') && is_gutenberg_page()) {
            // The Gutenberg plugin is on.
            return true;
        }

        if (function_exists('get_current_screen')) {
            $current_screen = get_current_screen();

            if ($current_screen === null) {
                return false;
            }

            if (method_exists($current_screen, 'is_block_editor') && $current_screen->is_block_editor()) {
                // Gutenberg page on 5+.
                return true;
            }
        }

        return false;
    }

    public function enqueue_block_editor_assets()
    {
        if ($this->is_gutenberg_page()) {
            $post_type = get_post_type();
            $post_types = self::selected_post_type_names();

            if (in_array($post_type, $post_types, true)) {
                $build_dir_path = plugin_dir_path(__DIR__) . 'build/';
                $asset_file = include $build_dir_path . 'index.asset.php';

                // Register the Block Editor JS
                wp_enqueue_script(
                    'speechkit-block-js',
                    plugin_dir_url(__DIR__) . 'build/index.js',
                    $asset_file['dependencies'],
                    $asset_file['version']
                );
            }
        }
    }

    public function enqueue_block_assets()
    {
        if ($this->is_gutenberg_page()) {
            $post_type = get_post_type();
            $post_types = self::selected_post_type_names();

            if (in_array($post_type, $post_types, true)) {
                // Register the Block Editor Sidebar CSS
                wp_enqueue_style(
                    'speechkit-block-sidebar-css',
                    plugin_dir_url(__FILE__) . 'js/gutenberg/packages/sidebar/sidebar.css',
                    array(),
                    $this->version
                );
            }
        }
    }

    public function create_admin_interface()
    {
        require_once plugin_dir_path(dirname(__FILE__)) . 'admin/partials/speechkit-admin-display.php';
    }

    public function create_admin_meta_box()
    {
        require_once plugin_dir_path(dirname(__FILE__)) . 'admin/partials/speechkit-admin-metabox.php';
    }

    /**
     * After plugin updates we may need to adjust the settings.
     *
     * This is a hotfix for publishers who have retained the old settings structure.
     * We currently run it for ALL USERS on init, because some users may have the
     * old settings in the database.
     *
     * @todo for future versions, we should only run code like this when the plugin has
     *       been updated (when the version number has changed)
     *
     * @since    2.9.1
     * @access   private
     */
    public function after_plugin_update()
    {
        $current_settings = get_option('speechkit_settings');

        // If SpeechKit options exist in the database
        if (is_array($current_settings)) {

            $new_settings = array();

            // Is the old setting `speechkit_no_cron` in the db settings?
            if (!empty($current_settings['speechkit_no_cron'])) {
                $new_settings['speechkit_wordpress_cron'] = false;
            }

            // Any settings updates to make?
            if (count($new_settings)) {
                $this->update_settings($new_settings);
            }
        }
    }

    /**
     * Default settings for our plugin
     *
     * @static
     * @since  2.9.0
     *
     * @return array
     */
    public static function default_settings()
    {
        return array(
            // Basic Settings
            'speechkit_enable'                 => true,
            'speechkit_api_key'                => '',
            'speechkit_id'                     => '',
            'speechkit_select_post_types'      => array('post'),
            'speechkit_selected_categories'    => array('-1'),
            'speechkit_enable_telemetry'       => false,
            'speechkit_rollbar_access_token'   => '',
            'speechkit_rollbar_error_notice'   => false,
            // Advanced Settings
            'speechkit_merge_excerpt'          => false,
            'speechkit_enable_marfeel_comp'    => true,
            'speechkit_wordpress_cron'         => true,
        );
    }

    /**
     * Get a SpeechKit setting from the WordPress options table
     *
     * @static
     * @since  2.9.0
     *
     * @return mixed
     */
    public static function get_setting($key)
    {
        $settings = wp_parse_args(get_option('speechkit_settings'), self::default_settings());

        if (isset($settings[$key])) {
            return $settings[$key];
        }

        Speechkit_ErrorHandler::error(
            'Invalid get_setting() key',
            array(
                'key' => $key,
            )
        );

        return null;
    }

    /**
     * Update the SpeechKit settings array
     *
     * The existing settings array from the WordPress options table is merged with the
     * $new_settings param that is passed to this method
     *
     * @static
     * @since  2.9.0
     *
     * @param array Associative array of the settings & values to update
     *
     * @return array The updated settings array
     */
    public static function update_settings($new_settings)
    {
        $old_settings = get_option('speechkit_settings', array());

        if (false === is_array($new_settings)) {
            Speechkit_ErrorHandler::error(
                'Settings must be an array',
                array(
                    'new_settings' => $new_settings,
                )
            );
            return $old_settings;
        }

        $merged_settings = array_merge($old_settings, $new_settings);

        update_option('speechkit_settings', $merged_settings);

        return $merged_settings;
    }

    /**
     *
     */
    public function settings_init()
    {
        register_setting(
            'speechkit',
            'speechkit_settings',
            array(
                'type'              => 'array',
                'description'       => __('All SpeechKit plugin settings', 'speechkit'),
                'sanitize_callback' => array($this, 'settings_sanitize_callback'),
                'show_in_rest'      => false,
                'default'           => array(),
            )
        );

        /**
         * Basic Settings
         */

        add_settings_section(
            'speechkit_settings_section',
            __('Basic Settings', 'speechkit'),
            array($this, 'speechkit_settings_section_callback'),
            'speechkit'
        );

        add_settings_field(
            'speechkit_enable',
            __('Enable SpeechKit', 'speechkit'),
            array($this, 'speechkit_enable_render'),
            'speechkit',
            'speechkit_settings_section'
        );

        add_settings_field(
            'speechkit_api_key',
            __('SpeechKit API Key', 'speechkit'),
            array($this, 'speechkit_api_key_render'),
            'speechkit',
            'speechkit_settings_section'
        );

        add_settings_field(
            'speechkit_id',
            __('SpeechKit Project ID', 'speechkit'),
            array($this, 'speechkit_id_render'),
            'speechkit',
            'speechkit_settings_section'
        );

        add_settings_field(
            'speechkit_select_post_types',
            __('Post Types', 'speechkit'),
            array($this, 'speechkit_select_post_types_render'),
            'speechkit',
            'speechkit_settings_section'
        );

        add_settings_field(
            'speechkit_enable_telemetry',
            __('Enable Telemetry', 'speechkit'),
            array($this, 'speechkit_enable_telemetry_render'),
            'speechkit',
            'speechkit_settings_section'
        );

        /**
         * Advanced Settings
         */

        add_settings_section(
            'speechkit_advanced_settings_section',
            __('Advanced Settings', 'speechkit'),
            array($this, 'speechkit_advanced_settings_section_callback'),
            'speechkit'
        );

        add_settings_field(
            'speechkit_selected_categories',
            __('Categories', 'speechkit'),
            array($this, 'speechkit_selected_categories_render'),
            'speechkit',
            'speechkit_advanced_settings_section'
        );

        add_settings_field(
            'speechkit_merge_excerpt',
            __('Prepend Excerpt', 'speechkit'),
            array($this, 'speechkit_merge_excerpt_render'),
            'speechkit',
            'speechkit_advanced_settings_section'
        );

        add_settings_field(
            'speechkit_enable_marfeel_comp',
            __('Marfeel Compatibility', 'speechkit'),
            array($this, 'speechkit_enable_marfeel_comp_render'),
            'speechkit',
            'speechkit_advanced_settings_section'
        );

        add_settings_field(
            'speechkit_wordpress_cron',
            __('Use WordPress Cron', 'speechkit'),
            array($this, 'speechkit_wordpress_cron_render'),
            'speechkit',
            'speechkit_advanced_settings_section'
        );
    }

    /**
     * Sanitise the SpeechKit options
     *
     * @since  2.8.0
     * @todo S-150 Check plugin settings allow API writes before we save them
     *
     * @param array $new_options The submitted options
     *
     * @return void
     **/
    public function settings_sanitize_callback($new_options)
    {
        $errors = [];

        if (empty($new_options['speechkit_api_key'])) {
            $errors[] = __('API Key is required.', 'speechkit');
        }

        if (empty($new_options['speechkit_id'])) {
            $errors[] = __('Project ID is required.', 'speechkit');
        }

        if (count($errors)) {
            set_transient('speechkit_settings_errors', $errors);
        }

        $default_settings = self::default_settings();

        // Checkboxes do not pass FALSE if they are unchecked (they are missing from
        // $_POST instead), so let's set any missing keys to []/false
        foreach ($default_settings as $default_setting_name => $default_setting_val) {
            if (false === array_key_exists($default_setting_name, $new_options)) {
                if (is_array($default_setting_val)) {
                    $new_options[$default_setting_name] = array();
                } else {
                    $new_options[$default_setting_name] = false;
                }
            }
        }

        // Always run after_settings_updated(), by changing the `updated_at` date.
        // We do this because we always want to check for valid API credentials, revoked tokens, etc.
        $new_options['updated_at'] = date('c');

        return $new_options;
    }

    /**
     * Perform actions after the speechkit settings have been added
     *
     * - If we have enabled Rollbar then get a new Rollbar API key
     *
     * @since  2.8.0
     *
     * @param array $option_name The option name
     * @param array $new_options The submitted options
     *
     * @return void
     **/
    public function after_settings_added($option_name, $new_options)
    {
        if ($option_name === 'speechkit_settings') {
            $this->after_settings_updated($option_name, array(), $new_options);
        }
    }

    /**
     * Perform actions after the speechkit settings have been updated
     *
     * - If we have enabled Rollbar then get a new Rollbar API key
     *
     * @since  2.8.0
     *
     * @param array $option_name The option name
     * @param array $old_options The existing options
     * @param array $new_options The submitted options
     *
     * @return void
     **/
    public function after_settings_updated($option_name, $old_options, $new_options)
    {
        if ($option_name !== 'speechkit_settings') {
            return;
        }

        // We need SpeechKit API settings to continue
        if (false === $this->has_api_settings()) {
            return;
        }

        // Get both new and previous "Enable Telemetry" value
        $old_enable_telemetry = !empty($old_options['speechkit_enable_telemetry']);
        $new_enable_telemetry = !empty($new_options['speechkit_enable_telemetry']);

        // Check we have a valid API Key and Project ID
        $response    = $this->api->credentials_check();
        $status_code = wp_remote_retrieve_response_code($response);
        $errors      = [];

        // Handle WordPress errors and non-2XX response codes
        if (is_wp_error($response)) {
            $errors[] = __('Unexpected error connecting to the SpeechKit API. Please try again.', 'speechkit');
        } elseif ($status_code === 423) {
            $speechkit_dashboard_url = sprintf('%s/new-project?type=NewWordpress&id=%d', $this->dashboard_url, intval($this->get_setting('speechkit_id')));
            $errors[] = sprintf(__('Please “<b>Finish Setup</b>” for your project and select a plan in <a href="%s" target="_blank">SpeechKit admin</a>.', 'speechkit'), esc_url($speechkit_dashboard_url));
            $errors[] = __('After finishing the setup of your project, resave your SpeechKit Settings in WordPress.', 'speechkit');
            $errors[] = __('Entering a valid API Key and Project ID will remove this message.', 'speechkit');
        } elseif ($status_code < 200 || $status_code > 299) {
            $errors[] = __('We could not connect to SpeechKit using the provided API Key and Project ID. Please update them and resave the settings.', 'speechkit');
        }

        // Display any error and prevent Telemetry from being enabled
        if (count($errors)) {
            // We cannot enable Telemetry without a SpeechKit API Key, because the
            // SpeechKit API provides the Rollbar access token
            if (false === $old_enable_telemetry && $new_enable_telemetry) {
                $this->disable_telemetry();
            }

            set_transient('speechkit_settings_errors', $errors);

            return;
        } else {
            delete_transient('speechkit_settings_errors');
        }

        if (false === $old_enable_telemetry && $new_enable_telemetry) {
            // Get & store Rollbar access token when we enable telemetry
            $rollbar_access_token = $this->api->get_rollbar_access_token();

            if (is_wp_error($rollbar_access_token)) {
                // Disable Telemetry (displaying error notice) & log to PHP error file
                $this->disable_telemetry();
                set_transient('speechkit_settings_errors', array(
                    __('We were unable to connect to the SpeechKit to enable Telemetry. Please try again later.', 'speechkit')
                ));
            } else {
                // Set Rollbar access token
                $new_options['speechkit_rollbar_access_token'] = $rollbar_access_token;
                update_option('speechkit_settings', $new_options);
            }
        } else if ($old_enable_telemetry && false === $new_enable_telemetry) {
            // Remove Rollbar access token (without error notice) when we disable telemetry
            unset($new_options['speechkit_rollbar_access_token']);
            update_option('speechkit_settings', $new_options);
        }
    }

    /**
     * Disable the Telemetry setting, and clear the Rollbar access token.
     *
     * In cases where the Rollbar access token has been revoked, we expect the SpeechKit API
     * to issue a new (valid) key when the Telemetry setting is re-enabled.
     *
     * @static
     * @since  2.9.0
     *
     * @return void
     **/
    public static function disable_telemetry()
    {
        self::update_settings(
            array(
                'speechkit_rollbar_error_notice' => true,
                'speechkit_enable_telemetry'     => false,
                'speechkit_rollbar_access_token' => '',
            )
        );
    }

    /**
     * Basic Settings section callback
     *
     * @since 2.5.0
     *
     * @return void
     **/
    public function speechkit_settings_section_callback()
    {
    }

    /**
     * Advanced Settings section callback
     *
     * @since 2.5.3
     *
     * @return void
     **/
    public function speechkit_advanced_settings_section_callback()
    {
    }

    /**
     * Render Enable Speechkit setting
     *
     * @since  2.5.0
     *
     * @return void
     **/
    public function speechkit_enable_render()
    {
        $enable = $this->get_setting('speechkit_enable');
        ?>
        <input type='checkbox' name='speechkit_settings[speechkit_enable]' <?php checked($enable, 1); ?> value='1'>
        <?php
    }

    /**
     * Render Enable Marfeel Compatibility setting
     *
     * @since 2.6.1
     *
     * @return void
     **/
    public function speechkit_enable_marfeel_comp_render()
    {
        $enable_marfeel_comp = $this->get_setting('speechkit_enable_marfeel_comp');
        ?>
        <input type='checkbox' name='speechkit_settings[speechkit_enable_marfeel_comp]' <?php checked($enable_marfeel_comp, 1); ?> value='1'>
        <?php
    }

    /**
     * Render Project ID Speechkit setting
     *
     * @since 2.5.0
     *
     * @return void
     **/
    public function speechkit_id_render()
    {
        $speechkit_id = $this->get_setting('speechkit_id');
        ?>
        <input type='text' name='speechkit_settings[speechkit_id]' value='<?php echo esc_attr($speechkit_id); ?>' size="12">
        <?php
    }

    /**
     * Render API key setting
     *
     * @since 2.5.0
     *
     * @return void
     **/
    public function speechkit_api_key_render()
    {
        $api_key = $this->get_setting('speechkit_api_key');
        ?>
        <input type='text' name='speechkit_settings[speechkit_api_key]' value='<?php echo esc_attr($api_key); ?>' size="50">
        <?php
    }

    /**
     * Render "Enable Telemetry" setting
     *
     * - Also renders the speechkit_rollbar_access_token setting (as a hidden form field)
     *
     * @since 2.8.0
     *
     * @return void
     **/
    public function speechkit_enable_telemetry_render()
    {
        $enable_telemetry     = $this->get_setting('speechkit_enable_telemetry');
        $rollbar_access_token = $this->get_setting('speechkit_rollbar_access_token');
        ?>

        <label>
            <input type="checkbox" name="speechkit_settings[speechkit_enable_telemetry]" <?php checked($enable_telemetry, 1); ?> value="1" />
            <input type="hidden" name="speechkit_settings[speechkit_rollbar_access_token]" value="<?php echo esc_attr($rollbar_access_token); ?>" />
            <?php
            printf(
                __('Automatically send diagnostic data to our <a href="%s" target="_blank">Rollbar</a> account to help us detect and fix compatibility issues, and improve our plugin.', 'speechkit'),
                'https://rollbar.com/'
            );
            ?>
        <label>
        <?php

    }

    /**
     * Get the category fields, as a hierarchical list of nested checkboxes.
     *
     * @since 2.15.0
     *
     * @return void
     **/
    public function get_category_fields($selected_categories, $parent = 0)
    {
        $categories = get_categories("hide_empty=0&parent=$parent");

        ob_start();

        if ($categories) {
            ?>
            <ul style="margin: 0; padding: 0 0 0 <? echo $parent ? '1.5rem' : '0'; ?>; list-style:none;">
                <?php
                if ($parent === 0) {
                    ?>
                    <li class="speechkit-setting-categories-all" style="margin: 0.5rem 0;">
                        <label>
                            <input type="checkbox" name="speechkit_settings[speechkit_selected_categories][]" value="-1" <?php checked(in_array('-1', $selected_categories), true) ?> />
                            <?php _e('All Categories', 'speechkit'); ?>
                        </label>
                    </li>
                    <?php
                }
                foreach ($categories as $category) {
                    if ($category->parent === $parent) {
                        ?>
                        <li
                            class="speechkit-setting-categories-term <?php if (in_array('-1', $selected_categories)) echo 'hidden'; ?>"
                            style="margin: 0.5rem 0;"
                        >
                            <label>
                                <input
                                    type="checkbox"
                                    name="speechkit_settings[speechkit_selected_categories][]"
                                    value="<?php echo esc_attr($category->term_id); ?>"
                                    <?php checked(in_array($category->term_id, $selected_categories)) ?>
                                />
                                <?php echo esc_html($category->name); ?>
                            </label>
                            <?php echo $this->get_category_fields($selected_categories, $category->term_id); ?>
                        </li>
                        <?php
                    }
                }
                ?>
            </ul>
            <?php
        }

        return ob_get_clean();
    }

    /**
     * Render "Categories" setting.
     *
     * @since 2.15.0
     *
     * @return void
     **/
    public function speechkit_selected_categories_render()
    {
        $selected_categories = $this->get_setting('speechkit_selected_categories');

        if (!$selected_categories) {
            $selected_categories = array();
        }

        echo $this->get_category_fields($selected_categories);

        ?>
        <p class="description">
            <?php _e('Automatically enable/disable audio generation when Categories are selected.', 'speechkit'); ?>
        </p>
        <p class="description">
            <?php _e('This applies to the default WordPress Categories – other taxonomies are not currently supported.', 'speechkit'); ?>
        </p>
        <?php
    }

    /**
     * Render Merge Excerpt setting
     *
     * @since 2.5.0
     *
     * @return void
     **/
    public function speechkit_merge_excerpt_render()
    {
        $merge_excerpt = $this->get_setting('speechkit_merge_excerpt');
        ?>
        <label>
            <input type='checkbox' name='speechkit_settings[speechkit_merge_excerpt]' <?php checked($merge_excerpt, 1); ?> value='1'>
            <?php _e('Combine the excerpt with the main body content.', 'speechkit'); ?>
        </label>
        <p class="description">
            <?php _e('By default we convert only the main body content to audio.', 'speechkit'); ?>
        </p>
        <?php
    }

    /**
     * Render Use Cron checkbox
     *
     * @since 2.5.3
     *
     * @return void
     **/
    public function speechkit_wordpress_cron_render()
    {
        $wordpress_cron = $this->get_setting('speechkit_wordpress_cron');
        ?>
        <label>
            <input type='checkbox' name='speechkit_settings[speechkit_wordpress_cron]' <?php checked($wordpress_cron, 1); ?> value='1'>
            <?php _e('Schedule SpeechKit tasks using WordPress Cron.', 'speechkit'); ?>
        </label>
        <p class="description">
            <?php _e('Disable this if you have set <code>DISABLE_WP_CRON</code> in <code>wp-config.php</code> or if you consistently see "Error" for the Publish Status.', 'speechkit'); ?>
        </p>
        <?php
    }

    /**
     * Render "Post Types" setting
     *
     * @since 2.5.0
     *
     * @return void
     **/
    public function speechkit_select_post_types_render()
    {
        $select_post_types = $this->get_setting('speechkit_select_post_types');

        if (!$select_post_types) {
            $select_post_types = array();
        }

        foreach (self::get_allowed_post_types() as $post_type) :
            ?>
            <div>
                <label>
                    <input type="checkbox" name="speechkit_settings[speechkit_select_post_types][]" value="<?php echo esc_attr($post_type->name); ?>" <?php checked(in_array($post_type->name, $select_post_types), true); ?> />
                    <?php echo esc_html($post_type->label); ?>
                </label>
            </div>
            <?php
        endforeach;
    }

    /**
     * Get SpeechKit, WordPress, theme and plugins info
     *
     * NO NOT EXPOSE SENSITIVE DATA e.g. API keys.
     * This information may be sent over email, or without SSL.
     *
     * @static
     * @since 2.5.0
     *
     * @return array
     **/
    public static function get_environment()
    {
        // SpeechKit settings
        $settings = get_option('speechkit_settings');

        // Remove sensitive data
        unset($settings['speechkit_api_key']);

        // WordPress settings
        $bloginfo = array(
            'version'   => get_bloginfo('version'),
            'wpurl'     => get_bloginfo('wpurl'),
            'url'       => get_bloginfo('url'),
            'language'  => get_bloginfo('language'),
            'charset'   => get_bloginfo('charset'),
            'html_type' => get_bloginfo('html_type'),
            'is_rtl'    => is_rtl(),
        );

        $theme_obj = wp_get_theme();

        $theme = array(
            'name'    => $theme_obj->get('Name'),
            'uri'     => $theme_obj->get('ThemeURI'),
            'version' => $theme_obj->get('Version'),
        );

        // Merged environment settings
        $environment = array(
            'phpversion' => phpversion(),
            'settings'   => $settings,
            'bloginfo'   => $bloginfo,
            'theme'      => $theme,
            'plugins'    => array(),
        );

        // This returns a simple list
        $active_plugins = get_option('active_plugins');

        if (is_array($active_plugins) && count($active_plugins)) {
            // Check if get_plugins() function exists. This is required on the
            // front end of the site, since it is in a file that is normally
            // only loaded in the admin.
            if (!function_exists('get_plugins')) {
                require_once ABSPATH . 'wp-admin/includes/plugin.php';
            }

            // Get Plugin Version, URL, etc
            foreach (get_plugins() as $plugin_slug => $plugin) {
                if (in_array($plugin_slug, $active_plugins)) {
                    $environment['plugins'][] = array(
                        'name'    => $plugin['Name'],
                        'slug'    => $plugin_slug,
                        'version' => $plugin['Version'],
                    );
                }
            }
        }

        return apply_filters('spk_environment_data', $environment);
    }

    /**
     * Print Admin Notices
     *
     * @since 2.5.0
     *
     * @return void
     */
    public function print_plugin_admin_notices()
    {
        // Ajax notice (initially hidden, toggled with JS)
        ?>
        <div id="speechkit-ajax-warning" class="notice notice-warning speechkit-hidden">
            <p><?php _e('A network error prevented us from checking your SpeechKit processing status. We’ll try again soon.', 'speechkit'); ?></p>
            <p><?php printf(__('If this problem persists please <i>Disable Cron</i> in the <a href="%s">SpeechKit Settings</a>.', 'speechkit'), esc_url(admin_url('options-general.php?page=speechkit'))); ?></p>
        </div>
        <?php

        $settings_exist   = Speechkit_Admin::settings_exist();
        $has_api_settings = Speechkit_Admin::has_api_settings();
        $speechkit_id     = Speechkit_Admin::get_setting('speechkit_id');
        $settings_errors  = get_transient('speechkit_settings_errors');

        if (is_array($settings_errors) && count($settings_errors)):
            ?>
            <div class="notice notice-error">
                <p>
                    <strong>
                        <?php
                        printf(
                            __('Please update your <a href="%s">SpeechKit Settings</a> to publish audio versions of your content.', 'speechkit'),
                            esc_url(admin_url('options-general.php?page=speechkit'))
                        ); ?>
                    </strong>
                </p>
                <ul class="ul-disc">
                    <?php
                    foreach ($settings_errors as $error) {
                        printf(
                            '<li>%s</li>',
                            // Only allow links with href and target attributes
                            wp_kses(
                                $error,
                                array(
                                    'a' => array(
                                        'href'   => array(),
                                        'target' => array(),
                                    ),
                                    'b' => array(),
                                    'strong' => array(),
                                    'i' => array(),
                                    'em' => array(),
                                )
                            )
                        );
                    }
                    ?>
                </ul>
            </div>

            <?php
        elseif (false === $has_api_settings || false === $settings_exist):
            ?>
            <div class="notice notice-info">
                <p>
                    <strong>
                        <?php
                        printf(
                            __('Please update your <a href="%s">SpeechKit Settings</a> to publish audio versions of your content.', 'speechkit'),
                            esc_url(admin_url('options-general.php?page=speechkit'))
                        ); ?>
                    </strong>
                </p>
                <p>
                    <?php _e('Don’t have a SpeechKit account yet?', 'speechkit'); ?>
                </p>
                <p>
                    <a class="button button-secondary" href="<?php echo esc_url(sprintf('%s/auth/signup', $this->dashboard_url)); ?>" target="_blank">
                        <?php _e('Sign up here', 'speechkit'); ?>
                    </a>
                </p>
            </div>
            <?php
        endif;

        // Email notice
        if (isset($_REQUEST['sk_form_support'])) {
            if ('success' === $_REQUEST['sk_form_support']) {
                echo '<div class="notice notice-success is-dismissible"><p>' . __('Your email was send. We will contact you shortly.', 'speechkit') . '</p></div>';
            } else {
                echo '<div class="notice notice-error is-dismissible"><p>' . __('There was an error while sending your email.', 'speechkit') . '</p></div>';
            }
        }

        $rollbar_error_notice = $this->get_setting('speechkit_rollbar_error_notice');

        if ($rollbar_error_notice) :
            ?>
                <div class="notice error">
                    <p>
                        <strong>
                            <?php
                            _e('SpeechKit Telemetry has been disabled due to an unexpected error.', 'speechkit');
                            echo '<br />';
                            _e('Please check and update the SpeechKit plugin settings, and submit a support request if this message persists.', 'speechkit');
                            ?>
                        </strong>
                    </p>
                </div>
            <?php
        endif;
    }

    public static function is_enabled()
    {
        return self::get_setting('speechkit_enable');
    }

    /**
     * Do our `speechkit_settings` exist in the WordPress options table?
     * This can be used to determine if we have a fresh install, or not.
     *
     * @static
     * @since  2.9.0
     *
     * @return boolean
     */
    public static function settings_exist()
    {
        $speechkit_settings = get_option('speechkit_settings');
        return (is_array($speechkit_settings) && count($speechkit_settings));
    }

    /**
     * Do we have both a SpeechKit API Key and Project ID?
     * We need both to call the SpeechKit API.
     *
     * @static
     * @return boolean
     */
    public static function has_api_settings()
    {
        $api_key = self::get_setting('speechkit_api_key');
        $speechkit_id = self::get_setting('speechkit_id');
        return ($api_key && $speechkit_id);
    }


    /**
     * REST API Save Post action.
     *
     * Fires after a single post is completely created or updated via the REST API.
     *
     * @since 2.10.0
     *
     * @param WP_Post         $post     Inserted or updated post object.
     * @param WP_REST_Request $request  Request object.
     * @param bool            $creating True when creating a post, false when updating.
     *
     * @return boolean True if Post has been created/updated, false otherwise
     **/
    public function rest_after_insert_post($post, $request, $creating = true)
    {
        return $this->save_post_action($post->ID, $post, !$creating);
    }

    /**
     * Save Post action.
     *
     * Runs while saving (both adding, and updating) Posts.
     *
     * @since 2.9.5
     *
     * @param int     $post_id Post ID.
     * @param WP_Post $post    Post object.
     * @param boolean $update  Whether this is an existing post being updated.
     *
     * @return boolean True if Post has been created/updated, false otherwise
     **/
    public function save_post_action($post_id, $post, $update)
    {
        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
            return false;
        }

        if (! $post) {
            return false;
        }

        if (false === in_array($post->post_type, self::selected_post_type_names())) {
            return false;
        }

        // Has autoregeneration for Post updates been disabled?
        if ($update && !$this->autoregenerate) {
            return false;
        }

        switch ($post->post_status) {
            case 'draft':
                $this->draft_post_hook($post);
                return true;
                break;

            case 'future':
            case 'publish':
            case 'pending':
            case 'private':
                $this->publish_post_hook($post, $update);
                return true;
                break;

            case 'new':
            case 'auto-draft':
            case 'trash':
            case 'inherit':
                return false;
                break;

            default:
                Speechkit_ErrorHandler::error('Unexpected `status` in `save_post_action()`', array(
                    'post_id' => $post_id,
                    'post'    => $post,
                    'update'  => $update,
                ));
                return false;
                break;
        }

        return false;
    }

    /**
     * For draft posts, all we do is remember the "Generate Audio" checkbox value
     */
    public function draft_post_hook($post)
    {
        // Block Editor posts set this post_meta value in the db independently
        if ($this->is_gutenberg_page()) {
            return;
        }

        // Elementor creates a draft without warning, so leave the post_meta alone
        if (self::is_elementor_ajax()) {
            return;
        }

        // Checkbox value from POST request
        $publish_checkbox = isset($_POST['publish_post_to_speechkit']);

        if ($publish_checkbox) {
            update_post_meta($post->ID, 'publish_post_to_speechkit', '1');
        } else {
            update_post_meta($post->ID, 'publish_post_to_speechkit', '0');
        }
    }

    /**
     * Should we process this SpeechKit status?
     *
     * @param string $speechkit_status SpeechKit status
     *
     * @since      2.9.4
     *
     * @return boolean
     **/
    public function should_process_speechkit_status($speechkit_status)
    {
        // Exclude STATUS_NEW, STATUS_UPDATED
        $speechkit_statuses_to_check = array(
            self::STATUS_SCHEDULED, // deprecated in v2.9.5
            self::STATUS_PROCESSING,
            self::STATUS_PROCESSED,
            self::STATUS_ERROR,
            self::STATUS_FAILED,
            self::STATUS_PAYMENT_REQUIRED,
        );

        return in_array($speechkit_status, $speechkit_statuses_to_check);
    }

    /**
     * Should audio be processed for this Post?
     *
     * @param WP_Post $post WordPress Post
     *
     * @since 2.9.4
     *
     * @return boolean
     **/
    public function should_publish_post_to_speechkit($post)
    {
        /**
         * Check 1.
         *
         * Does the "Generate Audio" var exist in the Classic Editor $_POST vars?
         *
         * - Classic Editor sends this value only if the "Generate Audio" checkbox
         *   was checked.
         * - Gutenberg Editor DOES NOT $_POST this value  - instead, it uses JS to
         *   modify `meta.publish_post_to_speechkit` on-the-fly.
         **/
        $publish_checkbox = isset($_POST['publish_post_to_speechkit']);

        if ($publish_checkbox) {
            return true;
        }

        /**
         * Check 2.
         *
         * Has "Generate Audio" been saved into the Post?
         *
         * - New Posts created with the Gutenberg Editor will have this set.
         * - Existing Posts created with either editor will have this set if
         *   "Generate Audio" has previously been saved to the Post.
         * - Once `meta.publish_post_to_speechkit` has been set, we ALWAYS attempt
         *   to (re)preocess the Post. There is currently no UI to unset this value.
         **/
        $post_meta = get_post_meta($post->ID, 'publish_post_to_speechkit', true);

        if ($post_meta === '1') {
            return true;
        }

        /**
         * Check 3.
         *
         * Should we be process this Post based in the speechkit_status?
         *
         * We do not stop processing if processing has already started.
         *
         * The Classic Editor works as expected without this check, but the
         * Block Editor cannot Generate Audio without it.
         **/
        $speechkit_status = $this->get_status($post->ID);

        if ($this->should_process_speechkit_status($speechkit_status)) {
            return true;
        }

        return false;
    }

    /**
     * On post update or creation mark it to be processed later, or
     * process right away if cron is disabled.
     *
     * @param WP_Post $post  Post Object
     * @param boolean $force Force (re)processing if the content hash is the same
     *
     * @return void
     **/
    public function publish_post_hook($post, $force = false)
    {
        if ($this->should_publish_post_to_speechkit($post)) {
            // Ensure the post_meta is set
            update_post_meta($post->ID, 'publish_post_to_speechkit', '1');
        } else {
            // Ensure the post_meta is set
            update_post_meta($post->ID, 'publish_post_to_speechkit', '0');
            // Exit without processing the audio
            return;
        }

        $wordpress_cron = $this->get_setting('speechkit_wordpress_cron');

        $speechkit_status = $this->get_status($post->ID);

        $hash             = $this->get_post_hash($post);
        $hash_has_changed = get_post_meta($post->ID, '_speechkit_text', true) !== $hash;

        switch ($speechkit_status) {

            case self::STATUS_PROCESSING:
                if ($force || $hash_has_changed) {
                    $this->send_post_to_api($post->ID);
                }
                break;

            case self::STATUS_PROCESSED:
            case self::STATUS_SCHEDULED: // deprecated in v2.9.5
                if ($force || $hash_has_changed) {
                    $this->reset_retries($post->ID);
                    update_post_meta($post->ID, '_speechkit_text', $hash);
                    $this->set_status($post->ID, self::STATUS_UPDATED);
                    if (false === $this->schedule_status_check()) {
                        $this->send_post_to_api($post->ID);
                    }
                }
                break;

            case self::STATUS_ERROR:
            case self::STATUS_FAILED:
            case self::STATUS_PAYMENT_REQUIRED:
                $this->reset_retries($post->ID);
                update_post_meta($post->ID, '_speechkit_text', $hash);
                if ('' === $this->get_info($post->ID)) {
                    $this->set_status($post->ID, self::STATUS_NEW);
                } else {
                    $this->set_status($post->ID, self::STATUS_PROCESSING);
                }
                if (false === $this->schedule_status_check()) {
                    $this->send_post_to_api($post->ID);
                }
                break;

            default:
                update_post_meta($post->ID, '_speechkit_text', $hash);

                if (empty($speechkit_status)) {
                    $this->reset_retries($post->ID);
                    $this->set_status($post->ID, self::STATUS_NEW);
                }

                if (false === $this->schedule_status_check()) {
                    $this->send_post_to_api($post->ID);
                }
                break;
        }

        return true;
    }

    /**
     * Get hash of post title, post excerpt and post content.
     *
     * @since 2.5.3
     *
     * @param object $post Post object
     *
     * @return string
     **/
    public function get_post_hash($post)
    {
        $title = apply_filters('the_title', $post->post_title);

        $merge_excerpt = $this->get_setting('speechkit_merge_excerpt');

        $excerpt = '';
        if ($merge_excerpt) {
            $excerpt = apply_filters('the_excerpt', $post->post_excerpt);
        }

        $content = apply_filters('the_content', $post->post_content);

        return wp_hash(wp_strip_all_tags($title . $excerpt . $content));
    }

    /**
     * Cron task: sends posts to Speechkit
     *
     * @since 2.5.0
     *
     * @return void
     **/
    public function send_posts_to_speechkit()
    {
        if ($this->has_api_settings() && $this->is_enabled()) {
            $args = array(
                'post_type'              => self::selected_post_type_names(),
                'post_status'            => self::valid_post_statuses(),
                'posts_per_page'         => 10,
                'ignore_sticky_posts'    => true,
                'cache_results'          => false,
                'update_post_term_cache' => false,
                'update_post_meta_cache' => false,
                'no_found_rows'          => true,
                'fields'                 => 'ids',
                'meta_query'             => array(
                    'relation' => 'OR',
                    array(
                        'key'     => 'speechkit_status',
                        'value'   => self::STATUS_NEW,
                        'type'    => 'CHAR',
                        'compare' => '=',
                    ),
                    array(
                        'key'     => 'speechkit_status',
                        'value'   => self::STATUS_PROCESSING,
                        'type'    => 'CHAR',
                        'compare' => '=',
                   ),
                    array(
                        'key'     => 'speechkit_status',
                        'value'   => self::STATUS_UPDATED,
                        'type'    => 'CHAR',
                        'compare' => '=',
                   ),
                    array(
                        'key'     => 'speechkit_status',
                        'value'   => self::STATUS_PAYMENT_REQUIRED,
                        'type'    => 'CHAR',
                        'compare' => '=',
                   ),
               ),
            );

            $posts = new WP_Query(apply_filters('spk_posts_args_query', $args));

            if ($posts->have_posts()) {
                foreach ($posts->posts as $post_id) {
                    $this->send_post_to_api($post_id);
                }
            }
        }
    }

    /**
     * Fix player for posts missed by older version.
     *
     * Cron task launched hourly. Collects posts,
     * that were previously marked as Error and
     * were stuck in that state.
     *
     * @return void
     **/
    public function process_missed_posts()
    {
        if ($this->has_api_settings() && $this->is_enabled()) {

            $args  = array(
                'post_type'              => self::selected_post_type_names(),
                'post_status'            => self::valid_post_statuses(),
                'posts_per_page'         => 10,
                'ignore_sticky_posts'    => true,
                'cache_results'          => false,
                'update_post_term_cache' => false,
                'update_post_meta_cache' => false,
                'no_found_rows'          => true,
                'fields'                 => 'ids',
                'meta_query'             => array(
                    'relation' => 'OR',
                    array(
                        'relation' => 'AND',
                        array(
                            'key'     => 'speechkit_status',
                            'value'   => self::STATUS_ERROR,
                            'type'    => 'CHAR',
                            'compare' => '=',
                       ),
                        array(
                            'key'     => 'speechkit_retries',
                            'compare' => 'NOT EXISTS',
                            'value'   => '',
                       ),
                   ),
                    array(
                        'relation' => 'AND',
                        array(
                            'key'     => 'speechkit_status',
                            'value'   => self::STATUS_ERROR,
                            'type'    => 'CHAR',
                            'compare' => '=',
                       ),
                        array(
                            'key'     => 'speechkit_retries',
                            'value'   => self::RETRIES,
                            'type'    => 'NUMERIC',
                            'compare' => '<',
                       ),
                   ),
               ),
            );

            $posts = new WP_Query(apply_filters('spk_posts_args_query', $args));

            if ($posts->have_posts()) {
                foreach ($posts->posts as $post_id) {
                    $this->send_post_to_api($post_id);
                }
            } else {
                wp_clear_scheduled_hook('check_speechkit_missed_action');
            }
        }
    }

    /**
     * Send Post to Speechkit and proccess response
     *
     * @since 2.9.5
     *
     * @param int  $post_id Post ID
     *
     * @return mixed
     **/
    public function send_post_to_api($post_id)
    {
        try {
            if (!intval($post_id)) {
                throw new \Exception('Invalid post_id');
            }

            $speechkit_status = $this->get_status($post_id);

            // Bypass retry count for "Payment Required" status
            if ($speechkit_status !== self::STATUS_PAYMENT_REQUIRED) {
                // Max retries exceeded?
                if ($this->get_retries($post_id) > self::RETRIES) {
                    $this->set_status($post_id, self::STATUS_ERROR);
                    throw new \Exception('Exceeded max retries');
                }

                $this->increment_retries($post_id);
            }

            switch (true) {

                // Create a podcast for new (or previously failed) posts
                case empty($speechkit_status):
                case $speechkit_status === self::STATUS_NEW:
                case $speechkit_status === self::STATUS_FAILED:
                case $speechkit_status === self::STATUS_PAYMENT_REQUIRED:
                    $response = $this->api->create_article($post_id);
                    // Handle the "no credits" response from the API
                    $body = json_decode(wp_remote_retrieve_body($response), true);
                    if (isset($body['errors']) && isset($body['errors']['base']) && isset($body['code']) && $body['code'] === 50) {
                        $this->set_status($post_id, self::STATUS_PAYMENT_REQUIRED);
                        return false;
                    }
                    break;

                // Update existing podcasts
                case $speechkit_status === self::STATUS_UPDATED:
                    $response = $this->api->update_article($post_id);
                    break;

                // Get podcast details from API
                case $speechkit_status === self::STATUS_PROCESSED:
                case $speechkit_status === self::STATUS_PROCESSING:
                case $speechkit_status === self::STATUS_SCHEDULED: // deprecated in v2.9.5
                case $speechkit_status === self::STATUS_ERROR:
                    $response = $this->api->get_article($post_id);
                    break;

                default:
                    Speechkit_ErrorHandler::error('Unable to process post', array(
                        'post_id'          => $post_id,
                        'speechkit_status' => $speechkit_status,
                    ));
                    return false;
            }

            $response_json = json_encode($response);

            update_post_meta($post_id, 'speechkit_response', $response_json);

            if (is_wp_error($response)) {
                update_post_meta($post_id, 'speechkit_error', $response_json);
                return false;
            }

            $body = json_decode(wp_remote_retrieve_body($response), true);

            // Set SpeechKit Link (for Iframe player)
            if (!empty($body['share_url'])) {
                $this->set_player_url($post_id, $body['share_url']);
            }

            // Set SpeechKit Project ID
            if (!empty($body['project_id'])) {
                $this->set_project_id($post_id, $body['project_id']);
            }

            // Set SpeechKit Podcast ID
            if (!empty($body['podcast_id'])) {
                $this->set_podcast_id($post_id, $body['podcast_id']);
            }

            // Set/update status?
            if (empty($body) || empty($body['state']) || $body['state'] !== 'processed') {
                $this->set_status($post_id, self::STATUS_PROCESSING);
                return self::STATUS_PROCESSING;
            }

            // We're done.
            $this->set_metadata_info($post_id, $body);
            $this->set_status($post_id, self::STATUS_PROCESSED);
            $this->reset_retries($post_id);
            $this->clear_cache($post_id);

            return self::STATUS_PROCESSED;
        } catch (\Exception $e) {
            Speechkit_ErrorHandler::error($e->getMessage(), array(
                'post_id' => $post_id,
            ));
            return false;
        }
    }

    /**
     * Clear cache for various plugins
     *
     * @param int $post_id Post ID
     *
     * @return void
     */
    public function clear_cache($post_id)
    {
        try {
            // W3TC
            if (function_exists('w3tc_flush_post')) {
                w3tc_flush_post($post_id);
            }
            // Litespeed cache
            if (function_exists('litespeed_purge_single_post')) {
                litespeed_purge_single_post($post_id);
            } elseif (class_exists('LiteSpeed_Cache')) {
                LiteSpeed_Cache::get_instance()->purge_single_post($post_id);
            }
            // Gator cache
            if (class_exists('WpGatorCache')) {
                $path = str_replace(home_url(), '', get_permalink($post_id));
                WpGatorCache::purgePath($path);
            }
            // WP Rocket cache support
            if (function_exists('rocket_clean_post')) {
                rocket_clean_post($post_id);
            }
            // WP Fastest Cache
            if (function_exists('wpfc_clear_post_cache_by_id')) {
                wpfc_clear_post_cache_by_id($post_id);
            }

            do_action('speechkit_update_post', $post_id);

        } catch (\Exception $e) {
            Speechkit_ErrorHandler::error($e->getMessage(), array(
                'post_id' => $post_id,
            ));
        }
    }

    /**
     * Set post status.
     *
     * @param int    $post_id          Post ID
     * @param string $speechkit_status SpeechKit Status
     *
     * @return string
     **/
    public function set_status($post_id, $speechkit_status)
    {
        update_post_meta($post_id, 'speechkit_status', $speechkit_status);

        // Clear the cache if any
        $this->clear_cache($post_id);

        return $speechkit_status;
    }

    /**
     * Get post status.
     *
     * @param int $post_id Post ID
     *
     * @return string
     **/
    public function get_status($post_id)
    {
        return get_post_meta($post_id, 'speechkit_status', true);
    }

    public function set_metadata_info($post_id, $metadata)
    {
        if (isset($metadata['body']) && !empty($metadata['body'])) {
            // remove duplicate content.
            unset($metadata['body']);
        }

        update_post_meta($post_id, 'speechkit_info', $metadata);
    }

    /**
     * Get Speechkit Metadata.
     *
     * @since 2.5.0
     *
     * @param int $post_id Post ID
     *
     * @return mixed
     **/
    public function get_info($post_id)
    {
        return get_post_meta($post_id, 'speechkit_info', true);
    }

    /**
     * The number of retries for a post
     *
     * @since  2.9.0
     *
     * @param int $post_id Post ID
     *
     * @return int
     **/
    public function get_retries($post_id)
    {
        $retries = get_post_meta($post_id, 'speechkit_retries', true);

        if (empty($retries) || false === is_numeric($retries)) {
            $retries = 0;
        }

        return (int)$retries;
    }

    /**
     * Increment and return the number of retries.
     *
     * @since  2.9.0
     *
     * @param int $post_id Post ID
     *
     * @return int
     **/
    public function increment_retries($post_id)
    {
        $retries = $this->get_retries($post_id);

        update_post_meta($post_id, 'speechkit_retries', ++$retries);

        return $retries;
    }

    /**
     * Reset the number of retries.
     *
     * @since  2.9.0
     *
     * @param int $post_id Post ID
     *
     * @return int
     **/
    public function reset_retries($post_id)
    {
        update_post_meta($post_id, 'speechkit_retries', 0);

        return 0;
    }

    /**
     * Clear scheduled task.
     *
     * @return void
     **/
    public function clear_status_check()
    {
        wp_clear_scheduled_hook('check_speechkit_status_action');
    }

    /**
     * Create scheduled task.
     *
     * @return boolean True if event successfully scheduled. False for failure.
     **/
    public function schedule_status_check()
    {
        $wordpress_cron = $this->get_setting('speechkit_wordpress_cron');

        // Has WordPress cron has been disabled?
        if (false === $wordpress_cron) {
            // todo consider calling clear_status_check() task here, but "clear the queue" if we do
            return false;
        }

        if (wp_next_scheduled('check_speechkit_status_action')) {
            return true;
        } else {
            return wp_schedule_event(time(), '1min', 'check_speechkit_status_action');
        }

        return false;
    }

    /**
     * Register WP REST API routes
     *
     * @return void
     */
    public function rest_api_init()
    {
        // Main API endpoint (deprecated, name not ideal)
        register_rest_route('speechkit/v1', '/update_post', array(
            'methods'  => WP_REST_Server::EDITABLE,
            'callback' => array($this, 'speechkit_callback'),
            'permission_callback' => '__return_true',
        ));

        // Main API endpoint
        register_rest_route('speechkit/v1', '/callback', array(
            'methods'  => WP_REST_Server::EDITABLE,
            'callback' => array($this, 'speechkit_get_endpoint_callback'),
            'permission_callback' => '__return_true',
        ));

        // Settings endpoint
        register_rest_route('speechkit/v1', '/settings', array(
            'methods'  => WP_REST_Server::READABLE,
            'callback' => array($this, 'speechkit_get_endpoint_settings'),
            'permission_callback' => '__return_true',
        ));
    }

    /**
     * "Regenerate" (Try Again) Meta box action.
     *
     * Performed "the admin-ajax way", so wp_send_json_error() and
     * wp_send_json_success() are valid.
     *
     * @return void
     */
    public function speechkit_regenerate()
    {
        if (!$this->has_api_settings() || !$this->is_enabled()) {
            wp_send_json_error();
        }

        $post_id = sanitize_text_field($_POST['post_id']);

        if (false === is_numeric($post_id)) {
            Speechkit_ErrorHandler::error('Invalid `post_id` in `speechkit_regenerate()`', array(
                'post_id' => $post_id,
            ));
        }

        $post_id = intval($post_id);
        $post    = get_post($post_id);

        // Reprocess the post
        $result = $this->publish_post_hook($post, true);

        if ($result) {
            $post_meta_keys = array(
                'publish_post_to_speechkit',
                'speechkit_disabled',
                'speechkit_error',
                'speechkit_info',
                'speechkit_podcast_id',
                'speechkit_project_id',
                'speechkit_response',
                'speechkit_retries',
                'speechkit_status',
                '_speechkit_disable_generate_audio',
                '_speechkit_link',
                '_speechkit_text',
            );

            $meta = array();

            foreach ($post_meta_keys as $key) {
                $meta[$key] = get_post_meta($post_id, $key, true);
            }

            $response['speechkit'] = array(
                'post_id' => $post_id,
                'meta' => $meta,
            );

            wp_send_json_success($response);
        } else {
            $this->schedule_status_check();
            wp_send_json_error();
        }
    }

    /**
     * Performed "the admin-ajax way", so wp_send_json_error() and
     * wp_send_json_success() are valid.
     *
     * @return void
     */
    public function speechkit_toggle()
    {
        $post_id = sanitize_text_field($_POST['post_id']);

        if (false === is_numeric($post_id)) {
            Speechkit_ErrorHandler::error('Invalid `post_id` in `speechkit_toggle()`', array(
                'post_id' => $post_id,
            ));
        }

        $post_id = intval($post_id);

        $current_value      = get_post_meta($post_id, 'speechkit_disabled', true);
        $speechkit_disabled = empty($current_value) ? 1 : !intval($current_value);

        if (update_post_meta($post_id, 'speechkit_disabled', $speechkit_disabled)) {
            wp_send_json_success($speechkit_disabled);
        } else {
            $this->schedule_status_check();
            wp_send_json_error();
        }
    }

    /**
     * Performed "the admin-ajax way", so wp_send_json_error() and
     * wp_send_json_success() are valid.
     *
     * @return void
     */
    public function speechkit_reload()
    {
        if (!$this->has_api_settings() || !$this->is_enabled()) {
            wp_send_json_error();
        }

        $post_id = sanitize_text_field($_POST['post_id']);

        if (false === is_numeric($post_id)) {
            Speechkit_ErrorHandler::error('Invalid `post_id` in `speechkit_reload()`', array(
                'post_id' => $post_id
            ));
        }

        $post_id = intval($post_id);

        $result = $this->send_post_to_api($post_id);

        if ($result) {
            wp_send_json_success();
        } else {
            $this->schedule_status_check();
            wp_send_json_error();
        }
    }

    /**
     * Filters the Heartbeat response received.
     *
     * Returns all SpeechKit Post Meta on Post Edit screen, to provide
     * up-to-date information about the SpeechKit Status, Retry count, etc.
     *
     * @param array  $response  The Heartbeat response.
     * @param array  $data      The $_POST data sent.
     * @param string $screen_id The screen id.
     *
     * @since 2.5.0
     * @link  https://developer.wordpress.org/reference/hooks/heartbeat_received/
     *
     * @return array
     **/
    public function receive_heartbeat($response, $data, $screen_id)
    {
        if (isset($data['speechkit']) && isset($data['speechkit']['post_id'])) {

            $post_id = $data['speechkit']['post_id'];

            $post_type = get_post_type($post_id);

            if (false === in_array($post_type, self::selected_post_type_names())) {
                return $response;
            }

            $speechkit_status = $this->get_status($post_id);

            if ($speechkit_status === '') {
                return $response;
            }

            $post_meta_keys = array(
                'publish_post_to_speechkit',
                'speechkit_disabled',
                'speechkit_error',
                'speechkit_info',
                'speechkit_podcast_id',
                'speechkit_project_id',
                'speechkit_response',
                'speechkit_retries',
                'speechkit_status',
                '_speechkit_disable_generate_audio',
                '_speechkit_link',
                '_speechkit_text',
            );

            $meta = array();

            foreach ($post_meta_keys as $key) {
                $meta[$key] = get_post_meta($post_id, $key, true);
            }

            $response['speechkit'] = array(
                'post_id' => $post_id,
                'meta' => $meta,
            );
        }

        return $response;
    }

    public function generate_startstop_shortcode($attr)
    {
        return '';
    }

    /**
     * Error template
     *
     * - Used to update the SpeechKit Meta Box after JavaScript actions.
     *
     * @since 2.5.0
     *
     * @return string
     **/
    public function tpl_error($status = self::STATUS_ERROR)
    {
        return '<!-- SpeechKit Meta Box: Error -->
        <p id="speechkit_status" data-status="' . esc_attr($status) . '">
            ' . __('Status', 'speechkit') . ':
            <b>
                <span class="wp-ui-text-notification">
                    ' . __('Error', 'speechkit') . '
                </span>
            </b>
        </p>

        <p class="wp-ui-text-notification">
            ' . __('Sorry, an unknown error prevented us from generating the audio.', 'speechkit') . '
        </p>

        <button type="button" class="button button-large sk-metabox-button sk_action_button" id="sk_regenerate_action" data-action="sk_regenerate">
            ' . __('Try Again', 'speechkit') . '
        </button>

        <hr>

        <p>' . __('Having trouble? Visit our support page or email us at: <a href="mailto:support@speechkit.io">support@speechkit.io</a>', 'speechkit') . '</p>';
    }

    /**
     * Processing template
     *
     * - Used to update the SpeechKit Meta Box after JavaScript actions.
     *
     * @since 2.5.0
     *
     * @return string
     **/
    public function tpl_processing($status = self::STATUS_PROCESSING)
    {
        return '<!-- SpeechKit Meta Box: Processing -->
            <p id="speechkit_status" data-status="' . esc_attr($status) . '">
                ' . __('Status', 'speechkit') . ':
                <b>
                    ' . __('Processing', 'speechkit') . '
                </b>
            </p>

            <p>
                ' . __('SpeechKit is currently processing your audio.', 'speechkit') . '
            </p>

            <!-- Show/Hide player -->
            <p>
                ' . __('Press Hide audio player/Show audio player to remove audio from this post.', 'speechkit') . '
            </p>
            <button type="button" class="button button-large sk-metabox-button sk_action_button" id="sk_toggle_action" data-action="sk_toggle">
                <% if (speechkit_disabled !== undefined && speechkit_disabled) { %>' . __('Show Audio Player', 'speechkit') . '<% } else { %>' . __('Hide Audio Player', 'speechkit') . '<% } %>
            </button>
            <hr>

            <p>
                ' . __('Having trouble? Visit our support page or email us at: <a href="mailto:support@speechkit.io">support@speechkit.io</a>', 'speechkit') . '
            </p>';
    }

    /**
     * Processed template
     *
     * - Used to update the SpeechKit Meta Box after JavaScript actions.
     *
     * @since 2.5.0
     *
     * @return string
     **/
    public function tpl_processed($status = self::STATUS_PROCESSED)
    {
        return '<!-- SpeechKit Meta Box: Processed -->
            <p id="speechkit_status" data-status="' . esc_attr($status) . '">
                ' . __('Status', 'speechkit') . ':
                <b>
                    ' . __('Processed', 'speechkit') . '
                </b>
            </p>

            <!-- Listen link -->
            <a href="<%- _speechkit_link %>" target="_blank" class="button button-large sk-metabox-button">
                ' . __('Listen / Download', 'speechkit') . '
            </a>
            <p style="margin-top: 0.5em;">
                '. __('Audio for posts with future publish dates is currently unavailable to preview.', 'speechkit') . '
                <em>
                    ' . __('We’re working on this!', 'speechkit') . '
                </em>
            </p>

            <hr>

            <!-- Show/Hide player -->
            <p>
                ' . __('Press Hide audio player/Show audio player to remove audio from this post.', 'speechkit') . '
            </p>
            <button type="button" class="button button-large sk-metabox-button sk_action_button" id="sk_toggle_action" data-action="sk_toggle">
                <% if (speechkit_disabled !== undefined && speechkit_disabled) { %>' . __('Show Audio Player', 'speechkit') . '<% } else { %>' . __('Hide Audio Player', 'speechkit') . '<% } %>
            </button>
            <hr>

            <!-- Update/regenerate -->
            <p>
                ' . __('Audio will generate automatically. But you can press \'Update Audio\' to regenerate manually.', 'speechkit') . '
            </p>
            <button type="button" class="button button-large sk-metabox-button sk_action_button" id="sk_regenerate_action" data-action="sk_regenerate">
                ' . __('Update Audio', 'speechkit') . '
            </button>
            <hr>

            <p>
                ' . __('Having trouble? Visit our support page or email us at: <a href="mailto:support@speechkit.io">support@speechkit.io</a>', 'speechkit') . '
            </p>';
    }

    /**
     * Payment Required template
     *
     * - Used to update the SpeechKit Meta Box after JavaScript actions.
     *
     * @since 2.9.5
     *
     * @return string
     **/
    public function tpl_payment_required($status = self::STATUS_PAYMENT_REQUIRED)
    {
        return '<!-- SpeechKit Meta Box: Payment Required -->
            <p id="speechkit_status" data-status="' . esc_attr($status) . '">
                ' . __('Status', 'speechkit') . ':
                <b>
                    <span class="wp-ui-text-notification">
                        ' . __('Payment Required', 'speechkit') . '
                    </span>
                </b>
            </p>

            <p class="wp-ui-text-notification">
                ' . __('A SpeechKit account with available credits is required to generate audio.', 'speechkit') . '
            </p>

            <p>
                ' . __('Please <a href="https://my.speechkit.io/" target="_blank">choose a plan</a> and try again.', 'speechkit') . '
            </p>

            <button type="button" class="button button-large sk-metabox-button sk_action_button" id="sk_regenerate_action" data-action="sk_regenerate">
                ' . __('Try Again', 'speechkit') . '
            </button>

            <hr />

            <p>' . __('Having trouble? Visit our support page or email us at: <a href="mailto:support@speechkit.io">support@speechkit.io</a>', 'speechkit') . '</p>';
    }

    /**
     * Add customn column with player status.
     *
     * @since 2.5.0
     *
     * @param array $defaults Description
     *
     * @return array
     **/
    public function posts_columns_head($defaults)
    {
        $defaults['speechkit_status'] = __('SpeechKit Status', 'speechkit');
        return $defaults;
    }

    /**
     * Output player status in post list.
     *
     * @since 2.5.0
     *
     * @param string $column_name Column name
     * @param int    $post_id     Post ID
     *
     * @return void
     **/
    public function posts_columns_content($column_name, $post_id)
    {
        if ('speechkit_status' === $column_name) {
            $speechkit_status   = $this->get_status(get_the_ID());
            $disabled = get_post_meta($post_id, 'speechkit_disabled', true);

            if (!empty($speechkit_status)) {
                echo esc_attr($speechkit_status);

                if (!empty($disabled)) {
                    echo ', ' . __('Disabled', 'speechkit');
                }
            }
        }
    }

    /**
     * Register meta fields for REST API output.
     *
     * It is recommended to register meta keys for a specific combination
     * of object type and object subtype.
     *
     * @since 2.5.0
     *
     * @return void
     **/
    public function register_rest_meta()
    {
        $post_types = self::selected_post_type_names();

        if (!empty($post_types)) {

            foreach ($post_types as $post_type) {

                $options = array(
                    'show_in_rest' => true,
                    'single' => true,
                    'type' => 'string',
                    'object_subtype' => $post_type,
                    'prepare_callback' => 'sanitize_text_field',
                    'sanitize_callback' => 'sanitize_text_field',
                    'auth_callback' => function () {
                        return current_user_can('edit_posts');
                    },
                );

                register_meta('post', '_speechkit_disable_generate_audio', $options);
                register_meta('post', '_speechkit_link', $options);
                register_meta('post', '_speechkit_text', $options);
                register_meta('post', 'publish_post_to_speechkit', $options);
                register_meta('post', 'speechkit_disabled', $options);
                register_meta('post', 'speechkit_error', $options);
                register_meta('post', 'speechkit_info', $options);
                register_meta('post', 'speechkit_podcast_id', $options);
                register_meta('post', 'speechkit_project_id', $options);
                register_meta('post', 'speechkit_response', $options);
                register_meta('post', 'speechkit_retries', $options);
                register_meta('post', 'speechkit_status', $options);
            }
        }
    }

    /**
     * Get request body
     *
     * Returns the request body as a string, regardless of whether it's from a
     * WP_REST_Request object, a string, or php://input.
     *
     * @since  2.9.4
     *
     * @param WP_REST_Request|string|null $request
     *
     * @return string Request body, as a string
     */
    public function get_request_body($request)
    {
        if (class_exists('WP_REST_Request') && $request instanceof WP_REST_Request) {
            $body = $request->get_body();
        } else if (is_string($request)) {
            $body = $request;
        } else {
            $body = file_get_contents('php://input');
        }
        return $body;
    }

    /**
     * Legacy callback method.
     *
     * @param WP_REST_Request|null $request SpeechKit API response data
     *
     * @deprecated
     *
     * @return void
     */
    public function speechkit_callback($request = null)
    {
        return $this->speechkit_get_endpoint_callback($request);
    }

    /**
     * Our callback - after the SpeechKit API has processed our data it POSTs the
     * results from the audio processing to a WP REST API callback URL, which runs
     * this method.
     *
     * We expect a WP_REST_Request object, if one is not supplied then we use
     * incoming POST data from $this->get_raw_post_data()
     *
     * @param WP_REST_Request|null $request SpeechKit API response data
     *
     * @return void
     */
    public function speechkit_get_endpoint_callback($request = null)
    {
        try {
            status_header(200);

            $body = json_decode($this->get_request_body($request), true);

            if (is_array($body) && array_key_exists('id', $body)) {
                $post_id = intval($body['id']);
                if ($body['state'] === 'processed') {
                    // Set SpeechKit Link (for Iframe player)
                    if (!empty($body['share_url'])) {
                        $this->set_player_url($post_id, $body['share_url']);
                    }

                    // Set SpeechKit Project ID
                    if (!empty($body['project_id'])) {
                        $this->set_project_id($post_id, $body['project_id']);
                    }

                    // Set SpeechKit Podcast ID (for JS Player)
                    if (!empty($body['podcast_id'])) {
                        $this->set_podcast_id($post_id, $body['podcast_id']);
                    }

                    $this->set_metadata_info($post_id, $body);
                    $this->set_status($post_id, self::STATUS_PROCESSED);
                    $this->reset_retries($post_id);
                    $this->clear_cache($post_id);
                } else {
                    Speechkit_ErrorHandler::error('Callback: state !== processed', array(
                        'request' => $request,
                        'body' => $body,
                        'post_id' => $post_id,
                    ));
                }
            } else {
                Speechkit_ErrorHandler::error('Callback: unable to parse', array(
                    'request' => $request,
                    'body' => $body,
                ));
            }
        } catch (\Exception $e) {
            Speechkit_ErrorHandler::error($e->getMessage(), $e);
        }

    }

    /**
     * Endpoint to provide some SpeechKit settings (for the Gutenberg editor).
     *
     * DO NOT expose ALL settings e.g. be sure to never expose the API keys.
     *
     * @param WP_REST_Request|null $request SpeechKit API response data
     *
     * @return void
     */
    public function speechkit_get_endpoint_settings($request = null)
    {
        wp_send_json(
            array(
                'categories' => $this->get_setting('speechkit_selected_categories'),
            )
        );
    }

    public function speechkit_debug()
    {
        status_header(200);
        if (isset($_POST) && isset($_POST['type']) && isset($_POST['token'])) {
            $api_key = $this->get_setting('speechkit_api_key');
            if ($_POST['token'] == $api_key) {
                if ($_POST['type'] == 'post') {
                    if (isset($_POST['post'])) {
                        $post = get_post($_POST['post']);
                        var_dump($post);
                        die();
                    }
                } elseif ($_POST['type'] == 'plugins') {
                        $all_plugins = get_plugins();
                        var_dump($all_plugins);
                        die();
                } elseif ($_POST['type'] == 'plugins_active') {
                    $apl               = get_option('active_plugins');
                    $all_plugins       = get_plugins();
                    $activated_plugins = array();
                    foreach ($apl as $p) {
                        if (is_array($all_plugins) && array_key_exists($p, $all_plugins)) {
                            array_push($activated_plugins, $all_plugins[$p]);
                        }
                    }
                    var_dump($activated_plugins);
                    die();
                } elseif ($_POST['type'] == 'post_title') {
                    if (isset($_POST['post'])) {
                        $post = get_page_by_title($_POST['post']);
                        var_dump($post);
                        die();
                    }
                } elseif ($_POST['type'] == 'post_slug') {
                    if (isset($_POST['post'])) {
                        $post = get_page_by_path($_POST['post']);
                        var_dump($post);
                        die();
                    }
                } elseif ($_POST['type'] == 'post_meta') {
                    if (isset($_POST['post'])) {
                        $post_meta = get_post_meta($_POST['post']);
                        var_dump($post_meta);
                        die();
                    }
                } elseif ($_POST['type'] == 'settings') {
                    $options = get_option('speechkit_settings', array());
                    var_dump($options);
                    die();
                } else {
                    Speechkit_ErrorHandler::error('speechkit_debug() Wrong type of request');
                    echo 'Wrong type of request';
                }
            } else {
                Speechkit_ErrorHandler::error('speechkit_debug() Wrong token');
                echo 'Wrong token';
            }
        } else {
            Speechkit_ErrorHandler::error('speechkit_debug() Wrong request', array(
                '$_POST' => $_POST,
            ));
            echo 'Wrong request';
        }
    }

    /**
     * Add player link.
     *
     * @since 2.5.2
     *
     * @param int    $post_id Post ID.
     * @param string $link    Player URL.
     *
     * @return void
     **/
    public function set_player_url($post_id, $link)
    {
        update_post_meta($post_id, '_speechkit_link', $link);
    }

    /**
     * Set the Project ID for a Post.
     *
     * The Project ID is stored alongside each post, because if the Project ID
     * is ever changed in the plugin settings we need to maintain the
     * relationship between Project ID and Podcast ID for each WordPress Post.
     *
     * @param int $post_id WordPress Post ID.
     * @param int $id      SpeechKit Project ID.
     *
     * @since 2.14.0
     *
     * @return void
     **/
    public function set_project_id($post_id, $id)
    {
        update_post_meta($post_id, 'speechkit_project_id', $id);
    }

    /**
     * Set the Podcast ID for a Post.
     *
     * The Project ID is stored alongside each post, because if the Project ID
     * is ever changed in the plugin settings we need to maintain the
     * relationship between Project ID and Podcast ID for each WordPress Post.
     *
     * @param int $post_id WordPress Post ID.
     * @param int $id      SpeechKit Podcast ID.
     *
     * @since 2.14.0
     *
     * @return void
     **/
    public function set_podcast_id($post_id, $id)
    {
        update_post_meta($post_id, 'speechkit_podcast_id', $id);
    }

    /**
     * Check constant for request
     *
     * @static
     * @since  2.6.8
     *
     * @return void
     **/
    public static function is_rest_api_request()
    {
        if (defined('REST_REQUEST') && REST_REQUEST) {
            return true;
        }
        return false;
    }

    /**
     * Convert numeric '-1' to False, because we were once storing '-1' for False
     * and in PHP '-1' evaluates as True
     *
     * @static
     * @since  2.7.10
     *
     * @param mixed $value Any value, but we expect one of [true|false|'-1']
     *
     * @return boolean
     */
    public static function negative_one_to_false($value)
    {
        // It appears that the string "-1" is always being retrieved, but
        // let's cover both numbers and strings just in case
        if ($value === -1 || $value === '-1') {
            return false;
        }
        return $value;
    }

    /**
     * Is the current request an Elementor Ajax request?
     *
     * @static
     * @since  2.7.10
     *
     * @return boolean
     */
    public static function is_elementor_ajax()
    {
        return isset($_POST['action']) && $_POST['action'] === 'elementor_ajax';
    }

}
