/* global ajaxurl, jQuery, speechkitData */

(function($) {
	'use strict';

	var speechkit = {

		init: function() {
			// Is Gutenberg active?
			if (speechkitData.gutenberg) {
				return;
			}

			this.setupSpeechkitData();
			this.setupClickEvents();
			this.setupHeartbeat();
			this.setupAutosaveVariables();
		},

		getPostId: function () {
			return $('#post_ID').val();
		},

		/**
		 * Setup (adjust) the window.speechkitData object.
		 *
		 * @since 2.15.0
		 *
		 * @return void
		 */
		setupSpeechkitData: function () {
			if (typeof speechkitData === 'object') {
				// Record the initial "Generate Audio" checkbox value on page load,
				// because it may be unchecked (if an existing post is being edited)
				speechkitData.publish_post_to_speechkit = $('input#publish_post_to_speechkit').prop('checked');
			}
		},

		/**
		 * Setup click events.
		 *
		 * @since 2.9.5
		 *
		 * @return void
		 */
		setupClickEvents: function () {
			var self = this;

			$('#speechkit').on('click', '.sk_action_button', function (event) {
				event.preventDefault();
				$(event.target).prop('disabled', true);
				var action = $(event.target).data('action');

				self.postAction(action).done(function (response) {
					$(event.target).removeProp('disabled');

					if (!response.success) {
						self.errorMsg();
					}

					// Show/Hide Player
					if (action === 'sk_toggle') {
						if (response.success && response.data) {
							$(event.target).html(speechkitData.disabled);
						} else if (response.success && !response.data) {
							$(event.target).html(speechkitData.enabled);
						}
					}

					// Regenerate Audio
					if (action === 'sk_regenerate') {
						self.render(response.data);
					}
				});
			});

			$(document).on('click', 'input[name="post_category[]"]', function (event) {
				var selectedCategories = speechkitData.selectedCategories;

				// Exit if SpeechKit is enabled for "All Categories"
				if ($.inArray('-1', selectedCategories) !== -1) {
					return;
				}

				// Get ALL currently-checked categories
				// var checkedCategories = $('input[name="post_category[]"]:checked');
				var checkedCategories = $('input[name="post_category[]"]:checked').map(function() {
					return this.value;
				}).get();

				// Reset to initial (page load) value if no categories are selected
				if (checkedCategories.length === 0 && typeof speechkitData === 'object') {
					$('input#publish_post_to_speechkit').prop('checked', speechkitData.publish_post_to_speechkit);
					return;
				}

				var intersections = checkedCategories.filter(e => selectedCategories.indexOf(e) !== -1);

				if (intersections.length) {
					$('input#publish_post_to_speechkit').prop('checked', true);
				} else {
					$('input#publish_post_to_speechkit').prop('checked', false);
				}
			});
		},

		/**
		 * Add our checkbox value to the autosave POST vars (if it's checked).
		 *
		 * @since 2.9.5
		 *
		 * @return void
		 */
		setupAutosaveVariables: function () {
			$(document).ajaxSend(function (event, request, settings) {
				var publish_post_to_speechkit = $('#publish_post_to_speechkit').is(':checked');
				if (publish_post_to_speechkit) {
					settings.data += '&' + $.param({
						publish_post_to_speechkit: '1'
					});
				}
			});
		},

		/**
		 * Update the Meta Box on WordPress heartbeat tick.
		 *
		 * @since 2.9.5
		 *
		 * @return void
		 */
		setupHeartbeat: function () {
			var self = this;

			// Hook into the heartbeat-send
			$(document).on('heartbeat-send', function (e, data) {
				data.speechkit = {
					post_id: self.getPostId(),
				};
				return data;
			});

			// When a heartbeat tick (or pulse) is received
			$(document).on('heartbeat-tick', function (event, data, status, xhr) {
				// Catch any non 2XX HTTP codes
				// e.g. 403 caused by Cloudflare security challenge
				if (xhr.status >= 200 && xhr.status <= 299) {
					$('#speechkit-ajax-warning').addClass('speechkit-hidden');
				} else {
					$('#speechkit-ajax-warning').removeClass('speechkit-hidden');
					return;
				}

				if (status) {
					self.render(data);
				}
			});
		},

		/**
		 *
		 */
		postAction: function (action) {
			var self = this;

			return $.post(ajaxurl, { post_id: self.getPostId(), action: action });
		},

		/**
		 *
		 */
		errorMsg: function() {
			alert(speechkitData.error);
		},

		/**
		 *
		 */
		render: function(data) {
			if (false === Object.prototype.hasOwnProperty.call(data, 'speechkit')) {
				return;
			}

			if (false === Object.prototype.hasOwnProperty.call(data.speechkit, 'meta')) {
				return;
			}

			if (false === Object.prototype.hasOwnProperty.call(data.speechkit.meta, 'speechkit_status')) {
				return;
			}

			var speechkit_status = data.speechkit.meta.speechkit_status;
			var template = _.template(speechkitData.tpl[speechkit_status]);
			var html = template(data.speechkit.meta);

			$('#speechkit').find('.inside').html(html);
		}
	}

	$(document).ready(function() {
		speechkit.init();
	})

})(jQuery);
