/**
 * WordPress dependencies
 */
import { __ } from '@wordpress/i18n';
import { CheckboxControl } from '@wordpress/components';
import { compose } from '@wordpress/compose';
import { withDispatch, withSelect } from '@wordpress/data';
import { Fragment, useEffect } from '@wordpress/element';

/**
 * Internal dependencies
 */
import GenerateAudioCheck from './check';

export function GenerateAudio( {
	onFirstRender,
	onUpdateGenerateAudio,
	generateAudio,
	wrapper,
} ) {
	const Wrapper = wrapper || Fragment;

	// Set the the `publish_post_to_speechkit` post meta when the component is
	// first rendered, to ensure that we save '1' instead of '' when the checkbox
	// is left untouched (checked)
	useEffect( () => onFirstRender( generateAudio ), [] );

	return (
		<GenerateAudioCheck>
			<Wrapper>
				<CheckboxControl
					className="speechkit--generate-audio"
					label={ __( 'Generate Audio', 'speechkit' ) }
					checked={ generateAudio }
					onChange={ () => {
						onUpdateGenerateAudio( ! generateAudio );
					} }
				/>
			</Wrapper>
		</GenerateAudioCheck>
	);
}

export default compose( [
	withSelect( ( select ) => {
		const { getEditedPostAttribute, getPostEdits } = select(
			'core/editor'
		);

		const { getGenerateAudioEdited } = select( 'speechkit/interactions' );
		const { getSettings } = select( 'speechkit/settings' );

		/**
		 * Get the Generate Audio value.
		 *
		 * This is a little complex because it is also controlled (auto-checked/unchecked)
		 * based on the assigned Categories, and we need to be able to override it.
		 */
		const getGenerateAudio = () => {
			const postEdits = getPostEdits();

			// Has Generate Audio been edited in this session (manually checked or unchecked)?
			if (
				getGenerateAudioEdited() &&
				typeof postEdits.meta !== 'undefined' &&
				typeof postEdits.meta.publish_post_to_speechkit !== 'undefined'
			) {
				return postEdits.meta.publish_post_to_speechkit !== '0';
			}

			if ( ! isUsingCategories() ) {
				return (
					getEditedPostAttribute( 'meta' )
						.publish_post_to_speechkit !== '0'
				);
			}

			if ( hasMatchingCategory() ) {
				return true;
			}

			return false;
		};

		/**
		 * The SpeechKit Status is stored in a WordPress Custom Field (post_meta) as
		 * a "normal text string", in English.For example, "Payment Required".
		 */
		const isUsingCategories = () => {
			const settings = getSettings();

			// Do we have settings?
			if ( ! settings ) {
				return false;
			}

			const settingsCategories = settings.categories || [];

			// Do we have valid categories from the plugin settings?
			if ( ! Array.isArray( settingsCategories ) ) {
				return false;
			}

			// Is "All Categories" selected in the plugin settings?
			if ( settingsCategories.includes( '-1' ) ) {
				return false;
			}

			return true;
		};

		/**
		 * The SpeechKit Status is stored in a WordPress Custom Field (post_meta) as
		 * a "normal text string", in English.For example, "Payment Required".
		 */
		const hasMatchingCategory = () => {
			const settings = getSettings();

			// Do we have settings?
			if ( ! settings ) {
				return false;
			}

			const settingsCategories = settings.categories || [];

			// Do we have valid categories from the plugin settings?
			if ( ! Array.isArray( settingsCategories ) ) {
				return false;
			}

			// Get the Post edits
			const postEdits = getPostEdits();

			// Have any categories been edited?
			if ( ! Array.isArray( postEdits.categories ) ) {
				return false;
			}

			// Get all Post categories
			const postCategories = getEditedPostAttribute( 'categories' );

			// Do any Post categories match the plugin settings?
			const hasMatchingCategories = postCategories.some( ( x ) => {
				return settingsCategories.includes( String( x ) );
			} );

			if ( hasMatchingCategories ) {
				return true;
			}

			return false;
		};

		return {
			generateAudio: getGenerateAudio(),
		};
	} ),
	withDispatch( ( dispatch ) => {
		const { editPost } = dispatch( 'core/editor' );
		const { setGenerateAudioEdited } = dispatch( 'speechkit/interactions' );

		return {
			onFirstRender: ( generateAudio ) => {
				// Update the Post Meta (AKA the Custom Field)
				editPost( {
					meta: {
						publish_post_to_speechkit: generateAudio ? '1' : '0',
					},
				} );
			},
			onUpdateGenerateAudio: ( generateAudio ) => {
				// Mark "Generate Audio" as (manually) edited, so the user can
				// override the "Categories" assignment
				setGenerateAudioEdited( true );

				// Update the Post Meta (AKA the Custom Field)
				editPost( {
					meta: {
						publish_post_to_speechkit: generateAudio ? '1' : '0',
					},
				} );
			},
		};
	} ),
] )( GenerateAudio );
