/**
 * External dependencies
 */
import dashify from 'dashify';

/**
 * WordPress dependencies
 */
import { __ } from '@wordpress/i18n';
import { compose } from '@wordpress/compose';
import { withSelect } from '@wordpress/data';
import { Fragment } from '@wordpress/element';

/**
 * Internal dependencies
 */
import * as SPEECHKIT_STATUS from '../../constants/speechkit-status';
import PostStatusSpinner from '../post-status-spinner';

export function StatusDescription( {
	status,
	description,
	wrapper = Fragment,
} ) {
	const Wrapper = wrapper;

	const classNames = [
		'speechkit-sidebar__post-status-description',
		`speechkit-sidebar__post-status-description--${ dashify( status ) }`,
	];

	return (
		description && (
			<Wrapper>
				<div>
					<PostStatusSpinner />
				</div>
				<div>
					<span className={ classNames.join( ' ' ) }>
						{ description }
					</span>
				</div>
			</Wrapper>
		)
	);
}

export default compose( [
	withSelect( ( select ) => {
		const { getEditedPostAttribute } = select( 'core/editor' );

		/**
		 * The SpeechKit Status is stored in a WordPress Custom Field (post_meta) as
		 * a "normal text string", in English.For example, "Payment Required".
		 */
		const getStatus = () => {
			return (
				getEditedPostAttribute( 'meta' ).speechkit_status ||
				SPEECHKIT_STATUS.NONE
			);
		};

		/**
		 * The SpeechKit Status description is optional.
		 */
		const getStatusDescription = () => {
			const status = getStatus();
			let description;

			switch ( status ) {
				case SPEECHKIT_STATUS.NEW:
				case SPEECHKIT_STATUS.UPDATED:
				case SPEECHKIT_STATUS.PROCESSING:
					description = __(
						'SpeechKit is currently processing your audio.',
						'speechkit'
					);
					break;

				case SPEECHKIT_STATUS.ERROR:
				case SPEECHKIT_STATUS.FAILED:
					description = __(
						'Sorry, an unknown error prevented us from generating the audio.',
						'speechkit'
					);
					break;

				case SPEECHKIT_STATUS.PAYMENT_REQUIRED:
					description = __(
						'A SpeechKit account with available credits is required to generate audio.',
						'speechkit'
					);
					break;

				default:
					description = '';
					break;
			}

			return description;
		};

		return {
			status: getStatus(),
			description: getStatusDescription(),
		};
	} ),
] )( StatusDescription );
