/**
 * External dependencies
 */
import dashify from 'dashify';
import { get, pick } from 'lodash';

/**
 * WordPress dependencies
 */
import { __ } from '@wordpress/i18n';
import { PanelBody, PanelRow } from '@wordpress/components';
import { compose, withInstanceId } from '@wordpress/compose';
import { withDispatch, withSelect } from '@wordpress/data';
import { Component } from '@wordpress/element';
import { addAction, removeAction } from '@wordpress/hooks';

/**
 * Internal dependencies
 */
import * as SPEECHKIT_STATUS from '../../constants/speechkit-status';
import PostDisplayPlayer from '../post-display-player';
import PostGenerateAudio from '../post-generate-audio';
import PostPaymentRequired from '../post-payment-required';
import PostPlayAudio from '../post-play-audio';
import PostRegenerateAudio from '../post-regenerate-audio';
import PostStatusDescription from '../post-status-description';

class PostStatusPanel extends Component {
	constructor() {
		super( ...arguments );

		this.sendHeartbeat = this.sendHeartbeat.bind( this );
		this.receiveHeartbeat = this.receiveHeartbeat.bind( this );
	}

	componentDidMount() {
		addAction(
			'heartbeat.tick',
			this.getHookName( 'tick' ),
			this.receiveHeartbeat
		);
		addAction(
			'heartbeat.send',
			this.getHookName( 'send' ),
			this.sendHeartbeat
		);
	}

	componentWillUnmount() {
		removeAction( 'heartbeat.tick', this.getHookName( 'tick' ) );
		removeAction( 'heartbeat.send', this.getHookName( 'send' ) );
	}

	/**
	 * Returns a `@wordpress/hooks` hook name specific to the instance of the
	 * component.
	 *
	 * @param {string} hookId Hook ID.
	 *
	 * @return {string} Hook name prefix.
	 */
	getHookName( hookId ) {
		const { instanceId } = this.props;
		return `speechkit/sidebar/post-status-panel-${ hookId }-${ instanceId }`;
	}

	/**
	 * Add Speechkit data to the "send" Heartbeat API requests.
	 *
	 * @param {Object} data Data to be sent in the heartbeat request
	 */
	sendHeartbeat( data ) {
		const { postId } = this.props;

		if ( postId ) {
			data.speechkit = {
				post_id: postId,
			};
		}

		return data;
	}

	/**
	 * Receive Speechkit data from the "tick" Heartbeat API responses.
	 *
	 * @param {Object} data Data received in the heartbeat request
	 */
	receiveHeartbeat( data ) {
		const { updateMeta } = this.props;

		const meta = get( data, [ 'speechkit', 'meta' ] );

		if ( ! meta ) {
			return;
		}

		updateMeta( meta );
	}

	render() {
		const { status, statusTitle } = this.props;

		const classNames = [
			'speechkit',
			'speechkit-sidebar__status',
			`speechkit-sidebar__status--${ dashify( status ) }`,
		];

		return (
			<PanelBody
				title={ `${ __( 'Status', 'speechkit' ) }: ${ statusTitle }` }
				opened={ true }
				className={ classNames.join( ' ' ) }
			>
				<PostStatusDescription wrapper={ PanelRow } />
				<PostPaymentRequired wrapper={ PanelRow } />
				<PostGenerateAudio wrapper={ PanelRow } />
				<PostPlayAudio wrapper={ PanelRow } />
				<PostDisplayPlayer wrapper={ PanelRow } />
				<PostRegenerateAudio wrapper={ PanelRow } />
			</PanelBody>
		);
	}
}

export default compose( [
	withSelect( ( select ) => {
		const { getEditedPostAttribute } = select( 'core/editor' );

		/**
		 * The SpeechKit Status is stored in a WordPress Custom Field (post_meta) as a
		 * "normal text string", in English. For example, "Payment Required".
		 */
		const getStatus = () =>
			getEditedPostAttribute( 'meta' ).speechkit_status ||
			SPEECHKIT_STATUS.NONE;

		/**
		 * The SpeechKit Status from the database isn't always exactly what we want to
		 * present to the admin user, so convert getStatus() to something more appropriate.
		 *
		 * The keys here need to match the Status constants in PHP:
		 *
		 * STATUS_NEW              = 'New';        // Presented as "Processing"
		 * STATUS_UPDATED          = 'Updated';    // Presented as "Processing"
		 * STATUS_SCHEDULED        = 'Scheduled';  // Presented as "Processed", deprecated in v2.9.5
		 * STATUS_PROCESSING       = 'Processing';
		 * STATUS_PROCESSED        = 'Processed';
		 * STATUS_ERROR            = 'Error';
		 * STATUS_FAILED           = 'Failed';
		 * STATUS_PAYMENT_REQUIRED = 'Payment Required';
		 */
		const getStatusTitle = () => {
			const status = getStatus();
			let title;

			switch ( status ) {
				// A status of "" (empty/missing) is presented as "New"
				case SPEECHKIT_STATUS.NONE:
					title = __( 'New', 'speechkit' );
					break;

				// "New", "Updated" and "Processing" are all presented as "Processing"
				case SPEECHKIT_STATUS.NEW:
				case SPEECHKIT_STATUS.UPDATED:
				case SPEECHKIT_STATUS.PROCESSING:
					title = __( 'Processing', 'speechkit' );
					break;

				// "Scheduled" and "Processed" are both presented as "Processed"
				case SPEECHKIT_STATUS.SCHEDULED:
				case SPEECHKIT_STATUS.PROCESSED:
					title = __( 'Processed', 'speechkit' );
					break;

				case SPEECHKIT_STATUS.ERROR:
					title = __( 'Error', 'speechkit' );
					break;

				case SPEECHKIT_STATUS.FAILED:
					title = __( 'Failed', 'speechkit' );
					break;

				case SPEECHKIT_STATUS.PAYMENT_REQUIRED:
					title = __( 'Payment Required', 'speechkit' );
					break;

				default:
					title = __( 'Unknown Status', 'speechkit' );
					break;
			}

			return title;
		};

		return {
			postId: getEditedPostAttribute( 'id' ),
			status: getStatus(),
			statusTitle: getStatusTitle(),
		};
	} ),
	withDispatch( ( dispatch ) => {
		const { editPost } = dispatch( 'core/editor' );
		return {
			updateMeta: ( meta ) => {
				// Whitelist SpeechKit meta
				meta = pick( meta, [
					'publish_post_to_speechkit',
					'speechkit_disabled',
					'speechkit_error',
					'speechkit_info',
					'speechkit_podcast_id',
					'speechkit_project_id',
					'speechkit_response',
					'speechkit_retries',
					'speechkit_status',
					'_speechkit_link',
					'_speechkit_text',
				] );
				editPost( { meta } );
			},
		};
	} ),
	withInstanceId,
] )( PostStatusPanel );
