<?php

/**
 * Fired during plugin activation
 *
 * @link       https://speechkit.io
 * @since      1.0.0
 *
 * @package    Speechkit
 * @subpackage Speechkit/includes
 */

/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 *
 * @since      1.0.0
 * @package    Speechkit
 * @subpackage Speechkit/includes
 * @author     Kostas Vaggelakos <kostas@speechkit.io>
 */
class Speechkit_Activator {

    /**
     * The version of this plugin.
     *
     * @since    2.5.0
     * @access   public
     * @var      string    $version    The current version of this plugin.
     */
    public static $version;

    public function __construct() {
        // 2.4.9 is for compatibility with versions prior 2.5.0
        self::$version = get_option( 'speechkit_version', '2.4.9' );
    }

    /**
     * Upgrade database on plugin update.
     *
     * - Migrating from a version <= 2.4.9 needs a lot of options renaming, because
     *   multiple options have been restructured into a single array
     *
     * - Migrating from a version between 2.5.0 <=> 2.8.3 has a couple of options to rename:
     *   1. `speechkit_disable_autopublish` -> `speechkit_generate_audio_default`
     *   2. `speechkit_no_cron` -> `speechkit_wordpress_cron`
     *   These new options are negated when they are set.
     *
     *
     * @return void
     * @since    2.5.0
     */
    public static function activate() {

        // Fresh install, or version prior to 2.5.0
        if ( version_compare( self::$version, '2.5.0', '<' ) ) {

            // Version prior to 2.5.0
            if ( get_option( 'sk_enabled' ) ) {
                $options = array(
                    // Existing options that need to be renamed
                    'speechkit_enable'            => get_option( 'sk_enabled' ),
                    'speechkit_id'                => get_option( 'sk_news_site_id' ),
                    'speechkit_api_key'           => get_option( 'sk_api_key' ),
                    'speechkit_merge_excerpt'     => get_option( 'sk_combine_excerpt_with_body' ),
                    'speechkit_select_post_types' => get_option( 'sk_post_types' ),

                    // New options that need defaults to be set
                    'speechkit_wordpress_cron'         => true,
                );

                update_option( 'speechkit_settings', $options, true );

                delete_option( 'sk_enabled' );
                delete_option( 'sk_news_site_id' );
                delete_option( 'sk_api_key' );
                delete_option( 'sk_backend' );
                delete_option( 'sk_analytics_id' );
                delete_option( 'sk_analytics_key' );
                delete_option( 'sk_player_version' );
                delete_option( 'sk_execution_order' );
                delete_option( 'sk_processing_posts' );
                delete_option( 'sk_player' );
                delete_option( 'sk_post_types' );
                delete_option( 'sk_combine_excerpt_with_body' );
            }
        // Version between 2.5.0 and 2.8.x
        } elseif ( version_compare( self::$version, '2.9.0', '<' ) ) {
            $options = get_option( 'speechkit_settings' );

            if (false === $options) {
                return;
            }

            if (isset($options['speechkit_disable_autopublish'])) {
                unset($options['speechkit_disable_autopublish']);
            }

            if (isset($options['speechkit_no_cron'])) {
                unset($options['speechkit_no_cron']);
            } else {
                $options['speechkit_wordpress_cron'] = true;
            }

            update_option( 'speechkit_settings', $options, true );
        }

        // Bump version number in database
        update_option( 'speechkit_version', Speechkit::get_plugin_version(), true );
    }

}
