<?php


class SpeechkitAPI
{
    function __construct($version, $project_id)
    {
        $this->version    = $version;
        $this->project_id = $project_id;

        // Default to live environment
        $this->backend = 'https://app.speechkit.io';

        // Define SPEECHKIT_API_URL in wp-config.php to use other environments
        if (defined('SPEECHKIT_API_URL') && filter_var(SPEECHKIT_API_URL, FILTER_SANITIZE_URL)) {
            $this->backend = SPEECHKIT_API_URL;
        }
    }

    public function generate_headers($allowed = array())
    {
        $api_key = Speechkit_Admin::get_setting('speechkit_api_key');

        $headers = array(
            'Authorization' => 'Token ' . $api_key,
            'Content-Type'  => 'application/json',
            'User-Agent'    => 'SpeechKit WP Plugin v' . $this->version,
        );

        if (is_array($allowed) && count($allowed)) {
            $headers = array_intersect_key($headers, array_flip($allowed));
        }

        return $headers;
    }

    public function generate_default_fetch_options($allowed_headers = array())
    {
        return array(
            'headers'     => $this->generate_headers($allowed_headers),
            'blocking'    => true,
            'timeout'     => 60,
            'httpversion' => '1.1',
        );
    }

    /**
     * Generate the HTTP body for outbound SpeechKit API create/update requests.
     *
     * @param int    $post_id WordPress Post ID.
     *
     * @since 2.9.5
     * @return mixed[] Key/value pairs of the HTTP body for the SpeechKit API.
     */
    public function get_api_request_body($post_id)
    {
        $post = get_post($post_id);

        // Construct the HTTP body to send to the SpeechKit API
        $body = array(
            'external_id'       => $post_id,
            'title'             => $post->post_title,
            'body'              => $this->get_post_body_content($post),
            'external_url'      => get_permalink($post_id),
            'published_at'      => $post->post_date_gmt,
            'author'            => get_the_author_meta('display_name', $post->post_author),
            'image_url'         => get_the_post_thumbnail_url($post_id, 'thumbnail'),
            'publisher_version' => $this->version,
            'latest_version'    => $this->get_latest_plugin_version(),
            'callback_url'      => get_rest_url(null, 'speechkit/v1/update_post'),
        );

        return $body;
    }

    /**
     * Get the Post "body" (main text content of the post), prepared and ready to be
     * sent off to the SpeechKit API.
     *
     * This is the:
     * - Main body content entered in WordPress
     * - Optionally filtered using [SpeechKit-Start]/[SpeechKit-Stop] "shortcodes"
     * - With all registered filters (from other plugins, etc) applied
     * - Optionally prepended with the Post excerpt
     * - Optionally filtered using the speechkit_content filter
     *
     * @param WP_Post $post The WordPress Post.
     *
     * @since 2.9.5
     * @return string The body (the processed $post->post_content).
     */
    public function get_post_body_content($post)
    {
        // Default to the Post content
        $content = $post->post_content;

        // If SpeechKit-Start/Stop tags are present then use the content within them
        $regex = '/\[SpeechKit-Start\](.*?)\[SpeechKit-Stop\]/s';

        if (preg_match_all($regex, $content, $match, PREG_PATTERN_ORDER) > 0) {
            $content = implode(' ', $match[1]);
        }

        // Apply standard WordPress filters to handle shortcodes etc
        $content = apply_filters('the_content', $content);

        // todo maybe we need to do something like this to deal with Shortcodes a little better?
        // $content = wp_strip_all_tags(apply_filters('the_content', $content));

        // Revert to the originally supplied content if we have nothing left
        if (empty($content)) {
            $content = $post->post_content;
        }

        // Optionally prepend the excerpt, if the setting is there
        $merge_excerpt = Speechkit_Admin::get_setting('speechkit_merge_excerpt');

        if ($merge_excerpt) {
            $content = $post->post_excerpt . '. ' . $content;
        }

        /**
         * Filters the content we send to the SpeechKit API.
         *
         * @since 2.10.1
         *
         * @param string $content The content we send to the SpeechKit API.
         */
        $content = apply_filters('speechkit_content', $content, $post->ID);

        return $content;
    }

    /**
     * Get latest plugin version from the WordPress Plugin repository.
     *
     * @since 2.9.5
     * @return string The latest SpeechKit WordPress Plugin version.
     */
    public function get_latest_plugin_version()
    {
        if (false === function_exists('plugins_api')) {
            include_once ABSPATH . 'wp-admin/includes/plugin-install.php';
        }

        $args = array(
            'slug' => 'speechkit',
            'fields' => array(
                'version' => true,
            )
        );

        $call_api = plugins_api('plugin_information', $args);

        $version = '';

        if (!is_wp_error($call_api)) {
            if (!empty($call_api->version)) {
                $version = $call_api->version;
            }
        }

        return $version;
    }

    public function generate_base_url()
    {
        return $this->backend . '/api/v2/projects/' . $this->project_id . '/';
    }

    /**
     * Process a response from wp_remote_get() or wp_remote_post()
     *
     * - Generate a Speechkit_ErrorHandler error for non-2XX responses
     *
     * Expected errors from the backend API are:
     * - 500 {code: 40, message: "Error. Please contact support@speechkit.io"}
     * - 401 {code: 20, message: "The authentication token was not recognised"}
     * - 401 {code: 60, message: "You don't have authorization to perform this action"}
     * - 422 {code: 70, message: "The audio is currently being processed, can't update now"}
     * - 402 {code: 60, error: "API access requires payment. Please enter payment information through the dashboard at: https://app.speechkit.io"}
     * - 400 {code: 30, error: "Invalid request"}
     * - 400 {code: 50, error: "Variety of validation errors"}
     * - 404 {code: 30, message: "Object not found"}
     *
     * @param mixed $response WP response object
     *
     * @since  2.8.0
     * @return mixed $response
     */
    private function _process_response($response)
    {
        $status_code = wp_remote_retrieve_response_code($response);

        // Success is always 2XX
        if ($status_code >= 200 && $status_code < 300) {
            return $response;
        }

        switch ($status_code) {
            // Expected errors to ignore
            case 400: // Variety of validation errors
            case 402: // API access requires payment
            case 422: // The audio is currently being processed, can't update now
                break;

            // Expected errors to log
            case 401: // Authentication errors
            case 404: // Object not found
            case 500: // Error. Please contact support@speechkit.io
                Speechkit_ErrorHandler::error('API Error Response', array(
                    'status_code' => $status_code,
                    'response' => $response,
                ));
                break;

            // Unexpected errors
            default:
                Speechkit_ErrorHandler::error('UNEXPECTED API Error Response', array(
                    'status_code' => $status_code,
                    'response' => $response
                ));
                break;
        }

        return $response;
    }

    /**
     * Get article (podcast)
     *
     * @return void
     */
    public function get_article( $post_id )
    {
        if (false === Speechkit_Admin::has_api_settings()) {
            return new WP_Error('speechkit_missing_creds', __('Missing SpeechKit API settings.', 'speechkit'));
        }

        $url = $this->generate_base_url() . 'podcasts/' . $post_id . '?cache=false';

        $response = wp_remote_get($url, $this->generate_default_fetch_options());

        return $this->_process_response($response);
    }

    /**
     * Create article (podcast)
     *
     * @return void
     */
    public function create_article($post_id)
    {
        if (false === Speechkit_Admin::has_api_settings()) {
            return new WP_Error('speechkit_missing_creds', __('Missing SpeechKit API settings.', 'speechkit'));
        }

        $body_from_post = $this->get_api_request_body($post_id);

        $url = $this->generate_base_url() . 'podcasts';

        $args = array_merge(
            $this->generate_default_fetch_options(),
            array(
                'method' => 'POST',
                'body'   => wp_json_encode($body_from_post),
            )
        );

        $response = wp_remote_post($url, $args);

        return $this->_process_response($response);
    }

    /**
     * Update article (podcast)
     *
     * @return void
     */
    public function update_article($post_id)
    {
        if (false === Speechkit_Admin::has_api_settings()) {
            return new WP_Error('speechkit_missing_creds', __('Missing SpeechKit API settings.', 'speechkit'));
        }

        $body_from_post = $this->get_api_request_body($post_id);

        $url = $this->generate_base_url() . 'podcasts/' . $post_id;

        $args = array_merge(
            $this->generate_default_fetch_options(),
            array(
                'method' => 'PUT',
                'body'   => wp_json_encode($body_from_post),
            )
        );

        $response = wp_remote_request($url, $args);

        return $this->_process_response($response);
    }

    /**
     * Credentials check for the SpeechKit API.
     *
     * Validate the Project ID and API Key options and return the API response.
     *
     * @since  2.9.0
     * @return void
     */
    public function credentials_check()
    {
        // Don't pass e.g. the Authorization header, we don't use it for this API call
        $allowed_headers = array(
            'Content-Type',
            'User-Agent',
        );

        $url = $this->backend . '/api/v2/system/credentials_check';

        $args = array_merge(
            $this->generate_default_fetch_options($allowed_headers),
            array(
                'method' => 'POST',
                'body'   => wp_json_encode(
                    array(
                        'api_key'    => Speechkit_Admin::get_setting('speechkit_api_key'),
                        'project_id' => Speechkit_Admin::get_setting('speechkit_id'),
                    )
                ),
            )
        );

        $response = wp_remote_post($url, $args);

        return $this->_process_response($response);
    }

    /**
     * GET a Rollbar access token from the SpeechKit API
     *
     * We need a Rollbar API key for error handling, and the API provides it
     * so we can revove and reissue keys without code changes.
     *
     * @since  2.8.0
     * @return void
     */
    public function get_rollbar_access_token()
    {
        if (false === Speechkit_Admin::has_api_settings()) {
            return new WP_Error('speechkit_missing_creds', __('Missing SpeechKit API settings.', 'speechkit'));
        }

        $url = $this->backend . '/api/v2/rollbar';

        $response = wp_remote_get($url, $this->generate_default_fetch_options());

        $body = wp_remote_retrieve_body($response);

        if (is_wp_error($body)) {
            return $body;
        }

        // Validate response format, we expect:
        // {"rollbar_access_token": "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"}
        $body = json_decode($body, true);
        if (is_array($body) && array_key_exists('rollbar_access_token', $body)) {
            return $body['rollbar_access_token'];
        }

        $wp_error = new WP_Error();
        $wp_error->add('speechkit_error', __('Invalid Rollbar access token response', 'speechkit'));
        return $wp_error;
    }

}
