<?php
/**
 * The file that defines the SpeechKit Utilities.
 *
 * These Utility methods are used across various classes and files.
 *
 * @package    Speechkit
 * @subpackage Speechkit/includes
 * @author     Stuart McAlpine <stu@speechkit.io>
 * @link       https://speechkit.io
 * @since      2.12.0
 */

/**
 * SpeechKit Utilities.
 *
 * @package    Speechkit
 * @subpackage Speechkit/includes
 * @author     Stuart McAlpine <stu@speechkit.io>
 * @since      2.12.0
 */
class Speechkit_Utils
{
    /**
     * Regular expression for pulling the Project ID and Podcast ID from the
     * SpeechKit API callback we have (hopefully) received and stored.
     *
     * In these responses the Project ID is quoted, but the Podcast ID has no quotes.
     * To allow for these inconsistencies (which may change over time) we allow
     * optional quotes (") for both Project ID and Podcast ID, using backreferences.
     *
     * @var string Regular expression for the callback response.
     *
     * @since 2.12.0
     */
    public static $callback_regex = '/{"id":(")?(\d+)(?(1)\1|),"podcast_id":(")?(\d+)(?(3)\3|)/';

    /**
     * Get the Podcast ID for a WordPress Post.
     *
     * Over time there have been various approaches to storing the Podcast ID.
     * This function tries each approach in reverse-date order.
     *
     * @param WP_Post $post WordPress Post
     *
     * @return mixed Podcast ID, or false
     */
    public static function get_podcast_id( $post )
    {
        if (!($post instanceof WP_Post)) {
            Speechkit_ErrorHandler::error('Speechkit_Utils::get_podcast_id() needs a WP_Post', array(
                'post' => $post,
            ));
            return false;
        }

        // First, check for the Podcast ID in it's own custom field
        $val = get_post_meta( $post->ID, 'speechkit_podcast_id', true );

        // It may also be found by parsing post_meta.speechkit_link
        if (!$val) {
            $speechkit_link = get_post_meta( $post->ID, 'speechkit_link', true );
            // Player URL can be either /a/[ID] or /e/[ID]
            preg_match('/\/[ae]\/(\d+)/', $speechkit_link, $matches);
            if ($matches) {
                $val = $matches[1];
            }
        }

        // It may also be found by parsing post_meta.speechkit_response
        if (!$val) {
            $speechkit_response = get_post_meta( $post->ID, 'speechkit_response', true );
            preg_match(Speechkit_Utils::$callback_regex, $speechkit_response, $matches);
            // $matches[4] is the Podcast ID (body.podcast_id)
            if ($matches && $matches[4]) {
                $val = $matches[4];
            }
        }

        /**
         * It may also be found by parsing post_meta.speechkit_info
         *
         * NOTE: This has been copied verbatim from the existing iframe player check
         *       at Speechkit_Public::iframe_player_embed_html(), in case it is
         *       needed for posts which were created a very long time ago.
         *       I cannot write unit tests for this to pass, they always fail for me,
         *       so there are currently no tests for it.
         **/
        if (!$val) {
            $article = get_post_meta( $post->ID, 'speechkit_info', true );
            if ( empty( $article ) || ! isset( $article['share_url'] ) ) {
                // This is exactly the same if/else statement that we have at
                // Speechkit_Public::iframe_player_embed_html(), but there is
                // nothing for us to to do here.
            } else {
                // This is the part that we need...
                $url = $article['share_url'];

                // Player URL can be either /a/[ID] or /e/[ID]
                preg_match('/\/[ae]\/(\d+)/', $url, $matches);
                if ($matches) {
                    $val = $matches[1];
                }
            }
        }

        if (!$val) {
            // throw Rollbar Error
            Speechkit_ErrorHandler::error('Unable to determine Podcast ID', array(
                'post' => $post,
                'val'  => $val,
            ));
        }

        return $val;
    }

    /**
     * Get the Project ID for a WordPress Post.
     *
     * It is possible to change the SpeechKit Project ID in the plugin settings,
     * so the current Project ID setting will not necessarily be correct for all
     * historic Posts. Because of this, we attempt to retrive the correct Project ID
     * from various custom fields, then fall-back to the plugin setting.
     *
     * @param WP_Post $post WordPress Post
     *
     * @return mixed Project ID, or false
     */
    public static function get_project_id( $post )
    {
        // The default (fall-back) is the plugin setting
        $default = Speechkit_Admin::get_setting('speechkit_id');

        if (!($post instanceof WP_Post)) {
            Speechkit_ErrorHandler::error('Speechkit_Utils::get_project_id needs a WP_Post', array(
                'post' => $post,
            ));
            return $default;
        }

        // First, check for the Project ID in it's own custom field
        $val = get_post_meta( $post->ID, 'speechkit_project_id', true );

        // It may also be found by parsing post_meta.speechkit_response
        if (!$val) {
            $speechkit_response = get_post_meta( $post->ID, 'speechkit_response', true );
            preg_match(Speechkit_Utils::$callback_regex, $speechkit_response, $matches);
            // $matches[2] is the Project ID (body.id)
            if ($matches && $matches[2]) {
                $val = $matches[2];
            }
        }

        // If we still don't have an ID then use the default from the plugin settings
        if (!$val) {
            $val = $default;
        }

        if (!$val) {
            // throw Rollbar Error
            Speechkit_ErrorHandler::error('Unable to determine Project ID', array(
                'post' => $post,
                'val'  => $val,
            ));
        }

        return $val;
    }
}